#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: postgresql_schema
short_description: Add or remove PostgreSQL schema
description:
- Add or remove PostgreSQL schema.
options:
  name:
    description:
    - Name of the schema to add or remove.
    required: true
    type: str
    aliases:
    - schema
  database:
    description:
    - Name of the database to connect to and add or remove the schema.
    type: str
    default: postgres
    aliases:
    - db
    - login_db
  owner:
    description:
    - Name of the role to set as owner of the schema.
    type: str
  session_role:
    description:
    - Switch to session_role after connecting.
    - The specified session_role must be a role that the current login_user is a member of.
    - Permissions checking for SQL commands is carried out as though the session_role
      were the one that had logged in originally.
    type: str
  state:
    description:
    - The schema state.
    type: str
    default: present
    choices: [ absent, present ]
  cascade_drop:
    description:
    - Drop schema with CASCADE to remove child objects.
    type: bool
    default: false
  ssl_mode:
    description:
      - Determines whether or with what priority a secure SSL TCP/IP connection will be negotiated with the server.
      - See U(https://www.postgresql.org/docs/current/static/libpq-ssl.html) for more information on the modes.
      - Default of C(prefer) matches libpq default.
    type: str
    default: prefer
    choices: [ allow, disable, prefer, require, verify-ca, verify-full ]
  ca_cert:
    description:
      - Specifies the name of a file containing SSL certificate authority (CA) certificate(s).
      - If the file exists, the server's certificate will be verified to be signed by one of these authorities.
    type: str
    aliases: [ ssl_rootcert ]
  trust_input:
    description:
    - If C(no), check whether values of parameters I(schema), I(owner), I(session_role) are potentially dangerous.
    - It makes sense to use C(no) only when SQL injections via the parameters are possible.
    type: bool
    default: yes
    version_added: '0.2.0'
seealso:
- name: PostgreSQL schemas
  description: General information about PostgreSQL schemas.
  link: https://www.postgresql.org/docs/current/ddl-schemas.html
- name: CREATE SCHEMA reference
  description: Complete reference of the CREATE SCHEMA command documentation.
  link: https://www.postgresql.org/docs/current/sql-createschema.html
- name: ALTER SCHEMA reference
  description: Complete reference of the ALTER SCHEMA command documentation.
  link: https://www.postgresql.org/docs/current/sql-alterschema.html
- name: DROP SCHEMA reference
  description: Complete reference of the DROP SCHEMA command documentation.
  link: https://www.postgresql.org/docs/current/sql-dropschema.html
author:
- Flavien Chantelot (@Dorn-) <contact@flavien.io>
- Thomas O'Donnell (@andytom)
extends_documentation_fragment:
- community.general.postgres

'''

EXAMPLES = r'''
- name: Create a new schema with name acme in test database
  community.general.postgresql_schema:
    db: test
    name: acme

- name: Create a new schema acme with a user bob who will own it
  community.general.postgresql_schema:
    name: acme
    owner: bob

- name: Drop schema "acme" with cascade
  community.general.postgresql_schema:
    name: acme
    state: absent
    cascade_drop: yes
'''

RETURN = r'''
schema:
  description: Name of the schema.
  returned: success, changed
  type: str
  sample: "acme"
queries:
  description: List of executed queries.
  returned: always
  type: list
  sample: ["CREATE SCHEMA \"acme\""]
'''

import traceback

try:
    from psycopg2.extras import DictCursor
except ImportError:
    # psycopg2 is checked by connect_to_db()
    # from ansible.module_utils.postgres
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.postgres import (
    connect_to_db,
    get_conn_params,
    postgres_common_argument_spec,
)
from ansible_collections.community.general.plugins.module_utils.database import (
    check_input,
    pg_quote_identifier,
    SQLParseError,
)
from ansible.module_utils._text import to_native

executed_queries = []


class NotSupportedError(Exception):
    pass


# ===========================================
# PostgreSQL module specific support methods.
#

def set_owner(cursor, schema, owner):
    query = 'ALTER SCHEMA %s OWNER TO "%s"' % (
            pg_quote_identifier(schema, 'schema'), owner)
    cursor.execute(query)
    executed_queries.append(query)
    return True


def get_schema_info(cursor, schema):
    query = ("SELECT schema_owner AS owner "
             "FROM information_schema.schemata "
             "WHERE schema_name = %(schema)s")
    cursor.execute(query, {'schema': schema})
    return cursor.fetchone()


def schema_exists(cursor, schema):
    query = ("SELECT schema_name FROM information_schema.schemata "
             "WHERE schema_name = %(schema)s")
    cursor.execute(query, {'schema': schema})
    return cursor.rowcount == 1


def schema_delete(cursor, schema, cascade):
    if schema_exists(cursor, schema):
        query = "DROP SCHEMA %s" % pg_quote_identifier(schema, 'schema')
        if cascade:
            query += " CASCADE"
        cursor.execute(query)
        executed_queries.append(query)
        return True
    else:
        return False


def schema_create(cursor, schema, owner):
    if not schema_exists(cursor, schema):
        query_fragments = ['CREATE SCHEMA %s' % pg_quote_identifier(schema, 'schema')]
        if owner:
            query_fragments.append('AUTHORIZATION "%s"' % owner)
        query = ' '.join(query_fragments)
        cursor.execute(query)
        executed_queries.append(query)
        return True
    else:
        schema_info = get_schema_info(cursor, schema)
        if owner and owner != schema_info['owner']:
            return set_owner(cursor, schema, owner)
        else:
            return False


def schema_matches(cursor, schema, owner):
    if not schema_exists(cursor, schema):
        return False
    else:
        schema_info = get_schema_info(cursor, schema)
        if owner and owner != schema_info['owner']:
            return False
        else:
            return True

# ===========================================
# Module execution.
#


def main():
    argument_spec = postgres_common_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True, aliases=['name']),
        owner=dict(type="str", default=""),
        database=dict(type="str", default="postgres", aliases=["db", "login_db"]),
        cascade_drop=dict(type="bool", default=False),
        state=dict(type="str", default="present", choices=["absent", "present"]),
        session_role=dict(type="str"),
        trust_input=dict(type="bool", default=True),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    schema = module.params["schema"]
    owner = module.params["owner"]
    state = module.params["state"]
    cascade_drop = module.params["cascade_drop"]
    session_role = module.params["session_role"]
    trust_input = module.params["trust_input"]

    if not trust_input:
        # Check input for potentially dangerous elements:
        check_input(module, schema, owner, session_role)

    changed = False

    conn_params = get_conn_params(module, module.params)
    db_connection = connect_to_db(module, conn_params, autocommit=True)
    cursor = db_connection.cursor(cursor_factory=DictCursor)

    try:
        if module.check_mode:
            if state == "absent":
                changed = not schema_exists(cursor, schema)
            elif state == "present":
                changed = not schema_matches(cursor, schema, owner)
            module.exit_json(changed=changed, schema=schema)

        if state == "absent":
            try:
                changed = schema_delete(cursor, schema, cascade_drop)
            except SQLParseError as e:
                module.fail_json(msg=to_native(e), exception=traceback.format_exc())

        elif state == "present":
            try:
                changed = schema_create(cursor, schema, owner)
            except SQLParseError as e:
                module.fail_json(msg=to_native(e), exception=traceback.format_exc())
    except NotSupportedError as e:
        module.fail_json(msg=to_native(e), exception=traceback.format_exc())
    except SystemExit:
        # Avoid catching this on Python 2.4
        raise
    except Exception as e:
        module.fail_json(msg="Database query failed: %s" % to_native(e), exception=traceback.format_exc())

    db_connection.close()
    module.exit_json(changed=changed, schema=schema, queries=executed_queries)


if __name__ == '__main__':
    main()



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.arista.eos.tests.unit.compat.mock import patch
from ansible_collections.arista.eos.plugins.modules import eos_eapi
from ansible_collections.arista.eos.tests.unit.modules.utils import (
    set_module_args,
)
from .eos_module import TestEosModule, load_fixture


class TestEosEapiModule(TestEosModule):

    module = eos_eapi

    def setUp(self):
        super(TestEosEapiModule, self).setUp()

        self.mock_run_commands = patch(
            "ansible_collections.arista.eos.plugins.modules.eos_eapi.run_commands"
        )
        self.run_commands = self.mock_run_commands.start()

        self.mock_load_config = patch(
            "ansible_collections.arista.eos.plugins.modules.eos_eapi.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_verify_state = patch(
            "ansible_collections.arista.eos.plugins.modules.eos_eapi.verify_state"
        )
        self.verify_state = self.mock_verify_state.start()

        self.command_fixtures = {}

    def tearDown(self):
        super(TestEosEapiModule, self).tearDown()

        self.mock_run_commands.stop()
        self.mock_load_config.stop()

        # hack for older version of mock
        # should be using patch.stopall() but CI is still failing
        try:
            self.mock_verify_state.stop()
        except RuntimeError:
            pass

    def load_fixtures(self, commands=None, transport="eapi"):
        def run_commands(module, commands, **kwargs):
            output = list()
            for cmd in commands:
                output.append(load_fixture(self.command_fixtures[cmd]))
            return output

        self.run_commands.side_effect = run_commands
        self.load_config.return_value = dict(diff=None, session="session")

    def start_configured(self, *args, **kwargs):
        self.command_fixtures = {
            "show vrf": "eos_eapi_show_vrf.text",
            "show management api http-commands | json": "eos_eapi_show_mgmt.json",
        }
        return self.execute_module(*args, **kwargs)

    def start_unconfigured(self, *args, **kwargs):
        self.command_fixtures = {
            "show vrf": "eos_eapi_show_vrf.text",
            "show management api http-commands | json": "eos_eapi_show_mgmt_unconfigured.json",
        }
        return self.execute_module(*args, **kwargs)

    def test_eos_eapi_http_enable(self):
        set_module_args(dict(http=True))
        commands = [
            "management api http-commands",
            "protocol http port 80",
            "no shutdown",
        ]
        self.start_unconfigured(changed=True, commands=commands)

    def test_eos_eapi_http_disable(self):
        set_module_args(dict(http=False))
        commands = ["management api http-commands", "no protocol http"]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_http_port(self):
        set_module_args(dict(http_port=81))
        commands = ["management api http-commands", "protocol http port 81"]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_http_invalid(self):
        set_module_args(dict(http_port=80000))
        self.start_unconfigured(failed=True)

    def test_eos_eapi_https_enable(self):
        set_module_args(dict(https=True))
        commands = [
            "management api http-commands",
            "protocol https port 443",
            "no shutdown",
        ]
        self.start_unconfigured(changed=True, commands=commands)

    def test_eos_eapi_https_disable(self):
        set_module_args(dict(https=False))
        commands = ["management api http-commands", "no protocol https"]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_https_port(self):
        set_module_args(dict(https_port=8443))
        commands = ["management api http-commands", "protocol https port 8443"]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_local_http_enable(self):
        set_module_args(dict(local_http=True))
        commands = [
            "management api http-commands",
            "protocol http localhost port 8080",
            "no shutdown",
        ]
        self.start_unconfigured(changed=True, commands=commands)

    def test_eos_eapi_local_http_disable(self):
        set_module_args(dict(local_http=False))
        commands = [
            "management api http-commands",
            "no protocol http localhost",
        ]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_local_http_port(self):
        set_module_args(dict(local_http_port=81))
        commands = [
            "management api http-commands",
            "protocol http localhost port 81",
        ]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_vrf(self):
        set_module_args(dict(vrf="test"))
        commands = [
            "management api http-commands",
            "no shutdown",
            "vrf test",
            "no shutdown",
        ]
        self.start_unconfigured(changed=True, commands=commands)

    def test_eos_eapi_change_from_default_vrf(self):
        set_module_args(dict(vrf="test"))
        commands = ["management api http-commands", "vrf test", "no shutdown"]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_default(self):
        set_module_args(dict())
        self.start_configured(changed=False, commands=[])

    def test_eos_eapi_vrf_missing(self):
        set_module_args(dict(vrf="missing"))
        self.start_unconfigured(failed=True)

    def test_eos_eapi_state_absent(self):
        set_module_args(dict(state="stopped"))
        commands = ["management api http-commands", "shutdown"]
        self.start_configured(changed=True, commands=commands)

    def test_eos_eapi_state_failed(self):
        self.mock_verify_state.stop()
        set_module_args(dict(state="stopped", timeout=1))
        result = self.start_configured(failed=True)
        "timeout expired before eapi running state changed" in result["msg"]



#!/usr/bin/python
# Copyright 2016 Google Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: gcpubsub_info
short_description: List Topics/Subscriptions and Messages from Google PubSub.
description:
    - List Topics/Subscriptions from Google PubSub.  Use the gcpubsub module for
      topic/subscription management.
      See U(https://cloud.google.com/pubsub/docs) for an overview.
    - This module was called C(gcpubsub_facts) before Ansible 2.9. The usage did not change.
requirements:
  - "python >= 2.6"
  - "google-auth >= 0.5.0"
  - "google-cloud-pubsub >= 0.22.0"
notes:
  - list state enables user to list topics or subscriptions in the project.  See examples for details.
author:
  - "Tom Melendez (@supertom) <tom@supertom.com>"
options:
  topic:
    type: str
    description:
       - GCP pubsub topic name.  Only the name, not the full path, is required.
    required: False
  view:
    type: str
    description:
       - Choices are 'topics' or 'subscriptions'
    choices: [topics, subscriptions]
    default: topics
  state:
    type: str
    description:
       - list is the only valid option.
    required: False
    choices: [list]
    default: list
  project_id:
    type: str
    description:
      - your GCE project ID
  credentials_file:
    type: str
    description:
      - path to the JSON file associated with the service account email
  service_account_email:
    type: str
    description:
      - service account email
'''

EXAMPLES = '''
- name: List all Topics in a project
  community.google.gcpubsub_info:
    view: topics
    state: list

- name: List all Subscriptions in a project
  community.google.gcpubsub_info:
    view: subscriptions
    state: list

- name: List all Subscriptions for a Topic in a project
  community.google.gcpubsub_info:
    view: subscriptions
    topic: my-topic
    state: list
'''

RETURN = '''
subscriptions:
    description: List of subscriptions.
    returned: When view is set to subscriptions.
    type: list
    sample: ["mysubscription", "mysubscription2"]
topic:
    description: Name of topic. Used to filter subscriptions.
    returned: Always
    type: str
    sample: "mytopic"
topics:
    description: List of topics.
    returned: When view is set to topics.
    type: list
    sample: ["mytopic", "mytopic2"]
'''

try:
    from ast import literal_eval
    HAS_PYTHON26 = True
except ImportError:
    HAS_PYTHON26 = False

try:
    from google.cloud import pubsub
    HAS_GOOGLE_CLOUD_PUBSUB = True
except ImportError as e:
    HAS_GOOGLE_CLOUD_PUBSUB = False

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.google.plugins.module_utils.gcp import check_min_pkg_version, get_google_cloud_credentials


def list_func(data, member='name'):
    """Used for state=list."""
    return [getattr(x, member) for x in data]


def main():
    module = AnsibleModule(argument_spec=dict(
        view=dict(choices=['topics', 'subscriptions'], default='topics'),
        topic=dict(required=False),
        state=dict(choices=['list'], default='list'),
        service_account_email=dict(),
        credentials_file=dict(),
        project_id=dict(), ),)
    if module._name in ('gcpubsub_facts', 'community.google.gcpubsub_facts'):
        module.deprecate("The 'gcpubsub_facts' module has been renamed to 'gcpubsub_info'",
                         version='3.0.0', collection_name='community.google')  # was Ansible 2.13

    if not HAS_PYTHON26:
        module.fail_json(
            msg="GCE module requires python's 'ast' module, python v2.6+")

    if not HAS_GOOGLE_CLOUD_PUBSUB:
        module.fail_json(msg="Please install google-cloud-pubsub library.")

    CLIENT_MINIMUM_VERSION = '0.22.0'
    if not check_min_pkg_version('google-cloud-pubsub', CLIENT_MINIMUM_VERSION):
        module.fail_json(msg="Please install google-cloud-pubsub library version %s" % CLIENT_MINIMUM_VERSION)

    mod_params = {}
    mod_params['state'] = module.params.get('state')
    mod_params['topic'] = module.params.get('topic')
    mod_params['view'] = module.params.get('view')

    creds, params = get_google_cloud_credentials(module)
    pubsub_client = pubsub.Client(project=params['project_id'], credentials=creds, use_gax=False)
    pubsub_client.user_agent = 'ansible-pubsub-0.1'

    json_output = {}
    if mod_params['view'] == 'topics':
        json_output['topics'] = list_func(pubsub_client.list_topics())
    elif mod_params['view'] == 'subscriptions':
        if mod_params['topic']:
            t = pubsub_client.topic(mod_params['topic'])
            json_output['subscriptions'] = list_func(t.list_subscriptions())
        else:
            json_output['subscriptions'] = list_func(pubsub_client.list_subscriptions())

    json_output['changed'] = False
    json_output.update(mod_params)
    module.exit_json(**json_output)


if __name__ == '__main__':
    main()



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
from ansible_collections.cisco.nxos.plugins.modules import nxos_vrf_af
from .nxos_module import TestNxosModule, load_fixture, set_module_args


class TestNxosVrfafModule(TestNxosModule):

    module = nxos_vrf_af

    def setUp(self):
        super(TestNxosVrfafModule, self).setUp()

        self.mock_load_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_vrf_af.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_vrf_af.get_config"
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestNxosVrfafModule, self).tearDown()
        self.mock_load_config.stop()
        self.mock_get_config.stop()

    def load_fixtures(self, commands=None, device=""):
        self.get_config.return_value = load_fixture(
            "nxos_vrf_af", "config.cfg"
        )
        self.load_config.return_value = None

    def test_nxos_vrf_af_present_current_non_existing(self):
        set_module_args(dict(vrf="vrf0", afi="ipv4", state="present"))
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["vrf context vrf0", "address-family ipv4 unicast"],
        )

    def test_nxos_vrf_af_present_current_existing(self):
        set_module_args(dict(vrf="vrf1", afi="ipv4", state="present"))
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_absent_current_non_existing(self):
        set_module_args(dict(vrf="vrf0", afi="ipv4", state="absent"))
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_absent_current_existing(self):
        set_module_args(dict(vrf="vrf1", afi="ipv4", state="absent"))
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["vrf context vrf1", "no address-family ipv4 unicast"],
        )

    def test_nxos_vrf_af_auto_evpn_route_target_present_current_existing(self):
        set_module_args(
            dict(vrf="vrf11", afi="ipv4", route_target_both_auto_evpn=True)
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_auto_evpn_route_target_present_current_non_existing(
        self
    ):
        set_module_args(
            dict(vrf="vrf10", afi="ipv4", route_target_both_auto_evpn=True)
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf10",
                "address-family ipv4 unicast",
                "route-target both auto evpn",
            ],
        )

    def test_nxos_vrf_af_auto_evpn_route_target_absent_current_existing(self):
        set_module_args(
            dict(vrf="vrf11", afi="ipv4", route_target_both_auto_evpn=False)
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf11",
                "address-family ipv4 unicast",
                "no route-target both auto evpn",
            ],
        )

    def test_nxos_vrf_af_auto_evpn_route_target_absent_current_non_existing(
        self
    ):
        set_module_args(
            dict(vrf="vrf1", afi="ipv4", route_target_both_auto_evpn=False)
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_import_present_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target import 65000:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_default_direction_present_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[{"rt": "65000:1000", "state": "present"}],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target import 65000:1000",
                "route-target export 65000:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_import_present_current_existing(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_default_direction_present_current_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[{"rt": "65000:1000", "state": "present"}],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_multi_import_present_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "import",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "import",
                        "state": "present",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target import 65000:1000",
                "route-target import 65001:1000",
                "route-target import 65002:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_import_present_current_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "import",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "import",
                        "state": "present",
                    },
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_import_absent_current_non_existing(self):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "absent",
                    }
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_import_absent_current_existing(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "absent",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target import 65000:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_import_absent_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_multi_import_absent_current_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target import 65000:1000",
                "no route-target import 65001:1000",
                "no route-target import 65002:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_import_absent_current_mix(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "import",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "import",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                    {
                        "rt": "65003:1000",
                        "direction": "import",
                        "state": "present",
                    },
                    {
                        "rt": "65004:1000",
                        "direction": "import",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target import 65002:1000",
                "route-target import 65003:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_export_present_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target export 65000:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_export_present_current_existing(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_multi_export_present_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "export",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "export",
                        "state": "present",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target export 65000:1000",
                "route-target export 65001:1000",
                "route-target export 65002:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_export_present_current_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "export",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "export",
                        "state": "present",
                    },
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_export_absent_current_non_existing(self):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "absent",
                    }
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_export_absent_current_existing(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "absent",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target export 65000:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_export_absent_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_multi_export_absent_current_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target export 65000:1000",
                "no route-target export 65001:1000",
                "no route-target export 65002:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_export_absent_current_mix(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "export",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "export",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                    {
                        "rt": "65003:1000",
                        "direction": "export",
                        "state": "present",
                    },
                    {
                        "rt": "65004:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target export 65002:1000",
                "route-target export 65003:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_both_present_current_non_existing(self):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target import 65000:1000",
                "route-target export 65000:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_both_present_current_existing(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_multi_both_present_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "both",
                        "state": "present",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target import 65000:1000",
                "route-target export 65000:1000",
                "route-target import 65001:1000",
                "route-target export 65001:1000",
                "route-target import 65002:1000",
                "route-target export 65002:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_both_present_current_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "both",
                        "state": "present",
                    },
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_both_absent_current_non_existing(self):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "absent",
                    }
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_both_absent_current_existing(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "absent",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target import 65000:1000",
                "no route-target export 65000:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_both_absent_current_non_existing(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_vrf_af_route_target_multi_both_absent_current_existing(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target import 65000:1000",
                "no route-target export 65000:1000",
                "no route-target import 65001:1000",
                "no route-target export 65001:1000",
                "no route-target import 65002:1000",
                "no route-target export 65002:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_both_absent_current_mix(self):
        set_module_args(
            dict(
                vrf="vrf21",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                    {
                        "rt": "65003:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65004:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf21",
                "address-family ipv4 unicast",
                "no route-target import 65002:1000",
                "no route-target export 65002:1000",
                "route-target import 65003:1000",
                "route-target export 65003:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_both_current_only_import_or_export(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf31",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65001:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {
                        "rt": "65002:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                    {
                        "rt": "65003:1000",
                        "direction": "both",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf31",
                "address-family ipv4 unicast",
                "route-target export 65000:1000",
                "route-target import 65001:1000",
                "no route-target import 65002:1000",
                "no route-target export 65003:1000",
            ],
        )

    def test_nxos_vrf_af_route_target_multi_direction_current_only_import_or_export(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf31",
                afi="ipv4",
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    },
                    {"rt": "65001:1000", "state": "present"},
                    {
                        "rt": "65002:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                    {
                        "rt": "65003:1000",
                        "direction": "export",
                        "state": "absent",
                    },
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf31",
                "address-family ipv4 unicast",
                "route-target export 65000:1000",
                "route-target import 65001:1000",
                "no route-target export 65003:1000",
            ],
        )

    def test_nxos_vrf_af_auto_evpn_route_target_and_manual_route_target(self):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                route_target_both_auto_evpn=True,
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "vrf context vrf1",
                "address-family ipv4 unicast",
                "route-target both auto evpn",
                "route-target import 65000:1000",
                "route-target export 65000:1000",
            ],
        )

    def test_nxos_vrf_af_auto_evpn_route_target_and_manual_route_targets_with_absent_vrf(
        self
    ):
        set_module_args(
            dict(
                vrf="vrf1",
                afi="ipv4",
                state="absent",
                route_target_both_auto_evpn=True,
                route_targets=[
                    {
                        "rt": "65000:1000",
                        "direction": "both",
                        "state": "present",
                    }
                ],
            )
        )
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            ["vrf context vrf1", "no address-family ipv4 unicast"],
        )



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: nxos_snmp_traps
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages SNMP traps.
description:
- Manages SNMP traps configurations.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- This module works at the group level for traps.  If you need to only enable/disable
  1 specific trap within a group, use the M(cisco.nxos.nxos_command) module.
- Be aware that you can set a trap only for an enabled feature.
options:
  group:
    description:
    - Case sensitive group.
    required: true
    choices:
    - aaa
    - bfd
    - bgp
    - bridge
    - callhome
    - cfs
    - config
    - eigrp
    - entity
    - feature-control
    - generic
    - hsrp
    - license
    - link
    - lldp
    - mmode
    - ospf
    - pim
    - rf
    - rmon
    - snmp
    - storm-control
    - stpx
    - switchfabric
    - syslog
    - sysmgr
    - system
    - upgrade
    - vtp
    - all
    type: str
  state:
    description:
    - Manage the state of the resource.
    required: false
    default: enabled
    choices:
    - enabled
    - disabled
    type: str
"""

EXAMPLES = """
# ensure lldp trap configured
- cisco.nxos.nxos_snmp_traps:
    group: lldp
    state: enabled

# ensure lldp trap is not configured
- cisco.nxos.nxos_snmp_traps:
    group: lldp
    state: disabled
"""

RETURN = """
commands:
    description: command sent to the device
    returned: always
    type: list
    sample: "snmp-server enable traps lldp ;"
"""


from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    nxos_argument_spec,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_capabilities,
)
from ansible.module_utils.basic import AnsibleModule


def get_platform_id(module):
    info = get_capabilities(module).get("device_info", {})
    return info.get("network_os_platform", "")


def execute_show_command(command, module):
    command = {"command": command, "output": "text"}

    return run_commands(module, command)


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_snmp_traps(group, module):
    body = execute_show_command("show run snmp all", module)[0].split("\n")

    resource = {}
    feature_list = [
        "aaa",
        "bfd",
        "bgp",
        "bridge",
        "callhome",
        "cfs",
        "config",
        "eigrp",
        "entity",
        "feature-control",
        "generic",
        "hsrp",
        "license",
        "link",
        "lldp",
        "mmode",
        "ospf",
        "pim",
        "rf",
        "rmon",
        "snmp",
        "storm-control",
        "stpx",
        "switchfabric",
        "syslog",
        "sysmgr",
        "system",
        "upgrade",
        "vtp",
    ]

    if "all" in group and "N3K-C35" in get_platform_id(module):
        module.warn(
            "Platform does not support bfd traps; bfd ignored for 'group: all' request"
        )
        feature_list.remove("bfd")

    for each in feature_list:
        for line in body:
            if each == "ospf":
                # ospf behaves differently when routers are present
                if "snmp-server enable traps ospf" == line:
                    resource[each] = True
                    break
            else:
                if "enable traps {0}".format(each) in line:
                    if "no " in line:
                        resource[each] = False
                        break
                    else:
                        resource[each] = True

    for each in feature_list:
        if resource.get(each) is None:
            # on some platforms, the 'no' cmd does not
            # show up and so check if the feature is enabled
            body = execute_show_command("show run | inc feature", module)[0]
            if "feature {0}".format(each) in body:
                resource[each] = False

    find = resource.get(group, None)

    if group == "all".lower():
        return resource
    elif find is not None:
        trap_resource = {group: find}
        return trap_resource
    else:
        # if 'find' is None, it means that 'group' is a
        # currently disabled feature.
        return {}


def get_trap_commands(group, state, existing, module):
    commands = []
    enabled = False
    disabled = False

    if group == "all":
        if state == "disabled":
            for feature in existing:
                if existing[feature]:
                    trap_command = "no snmp-server enable traps {0}".format(
                        feature
                    )
                    commands.append(trap_command)

        elif state == "enabled":
            for feature in existing:
                if existing[feature] is False:
                    trap_command = "snmp-server enable traps {0}".format(
                        feature
                    )
                    commands.append(trap_command)

    else:
        if group in existing:
            if existing[group]:
                enabled = True
            else:
                disabled = True

            if state == "disabled" and enabled:
                commands.append(
                    ["no snmp-server enable traps {0}".format(group)]
                )
            elif state == "enabled" and disabled:
                commands.append(["snmp-server enable traps {0}".format(group)])
        else:
            module.fail_json(
                msg="{0} is not a currently " "enabled feature.".format(group)
            )

    return commands


def main():
    argument_spec = dict(
        state=dict(choices=["enabled", "disabled"], default="enabled"),
        group=dict(
            choices=[
                "aaa",
                "bfd",
                "bgp",
                "bridge",
                "callhome",
                "cfs",
                "config",
                "eigrp",
                "entity",
                "feature-control",
                "generic",
                "hsrp",
                "license",
                "link",
                "lldp",
                "mmode",
                "ospf",
                "pim",
                "rf",
                "rmon",
                "snmp",
                "storm-control",
                "stpx",
                "switchfabric",
                "syslog",
                "sysmgr",
                "system",
                "upgrade",
                "vtp",
                "all",
            ],
            required=True,
        ),
    )

    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )

    warnings = list()
    results = {"changed": False, "commands": [], "warnings": warnings}

    group = module.params["group"].lower()
    state = module.params["state"]

    existing = get_snmp_traps(group, module)

    commands = get_trap_commands(group, state, existing, module)
    cmds = flatten_list(commands)
    if cmds:
        results["changed"] = True
        if not module.check_mode:
            load_config(module, cmds)

        if "configure" in cmds:
            cmds.pop(0)
        results["commands"] = cmds

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: pn_port_config
author: "Pluribus Networks (@rajaspachipulusu17)"
short_description: CLI command to modify port-config
description:
  - This module can be used to modify a port configuration.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.
    required: False
    type: str
  state:
    description:
      - State the action to perform. Use C(update) to modify the port-config.
    required: True
    type: str
    choices: ['update']
  pn_intf:
    description:
      - physical interface.
    required: False
    type: str
  pn_crc_check_enable:
    description:
      - CRC check on ingress and rewrite on egress.
    required: False
    type: bool
  pn_dscp_map:
    description:
      - DSCP map name to enable on port.
    required: False
    type: str
  pn_autoneg:
    description:
      - physical port autonegotiation.
    required: False
    type: bool
  pn_speed:
    description:
      - physical port speed.
    required: False
    choices: ['disable', '10m', '100m', '1g',
              '2.5g', '10g', '25g', '40g', '50g', '100g']
  pn_port:
    description:
      - physical port.
    required: False
    type: str
  pn_vxlan_termination:
    description:
      - physical port vxlan termination setting.
    required: False
    type: bool
  pn_pause:
    description:
      - physical port pause.
    required: False
    type: bool
  pn_loopback:
    description:
      - physical port loopback.
    required: False
    type: bool
  pn_loop_vlans:
    description:
      - looping vlans.
    required: False
    type: str
  pn_routing:
    description:
      - routing.
    required: False
    type: bool
  pn_edge_switch:
    description:
      - physical port edge switch.
    required: False
    type: bool
  pn_enable:
    description:
      - physical port enable.
    required: False
    type: bool
  pn_description:
    description:
      - physical port description.
    required: False
    type: str
  pn_host_enable:
    description:
      - Host facing port control setting.
    required: False
    type: bool
  pn_allowed_tpid:
    description:
      - Allowed TPID in addition to 0x8100 on Vlan header.
    required: False
    type: str
    choices: ['vlan', 'q-in-q', 'q-in-q-old']
  pn_mirror_only:
    description:
      - physical port mirror only.
    required: False
    type: bool
  pn_reflect:
    description:
      - physical port reflection.
    required: False
    type: bool
  pn_jumbo:
    description:
      - jumbo frames on physical port.
    required: False
    type: bool
  pn_egress_rate_limit:
    description:
      - max egress port data rate limit.
    required: False
    type: str
  pn_eth_mode:
    description:
      - physical Ethernet mode.
    required: False
    choices: ['1000base-x', 'sgmii', 'disabled', 'GMII']
  pn_fabric_guard:
    description:
      - Fabric guard configuration.
    required: False
    type: bool
  pn_local_switching:
    description:
      - no-local-switching port cannot bridge traffic to
        another no-local-switching port.
    required: False
    type: bool
  pn_lacp_priority:
    description:
      - LACP priority from 1 to 65535.
    required: False
    type: str
  pn_send_port:
    description:
      - send port.
    required: False
    type: str
  pn_port_mac_address:
    description:
      - physical port MAC Address.
    required: False
    type: str
  pn_defer_bringup:
    description:
      - defer port bringup.
    required: False
    type: bool
'''

EXAMPLES = """
- name: Port config modify
  community.network.pn_port_config:
    pn_cliswitch: "sw01"
    state: "update"
    pn_port: "all"
    pn_dscp_map: "foo"

- name: Port config modify
  community.network.pn_port_config:
    pn_cliswitch: "sw01"
    state: "update"
    pn_port: "all"
    pn_host_enable: true
"""

RETURN = """
command:
  description: the CLI command run on the target node.
  returned: always
  type: str
stdout:
  description: set of responses from the port-config command.
  returned: always
  type: list
stderr:
  description: set of error responses from the port-config command.
  returned: on error
  type: list
changed:
  description: indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.netvisor.pn_nvos import pn_cli, run_cli, booleanArgs
from ansible_collections.community.network.plugins.module_utils.network.netvisor.netvisor import run_commands


def check_cli(module, cli):
    """
    This method checks for idempotency using the dscp-map-show name command.
    If a user with given name exists, return True else False.
    :param module: The Ansible module to fetch input parameters
    :param cli: The CLI string
    """
    name = module.params['pn_dscp_map']

    cli += ' dscp-map-show name %s format name no-show-headers' % name
    out = run_commands(module, cli)[1]

    out = out.split()

    return True if name in out[-1] else False


def main():
    """ This section is for arguments parsing """

    state_map = dict(
        update='port-config-modify'
    )

    module = AnsibleModule(
        argument_spec=dict(
            pn_cliswitch=dict(required=False, type='str'),
            state=dict(required=True, type='str',
                       choices=['update']),
            pn_intf=dict(required=False, type='str'),
            pn_crc_check_enable=dict(required=False, type='bool'),
            pn_dscp_map=dict(required=False, type='str'),
            pn_autoneg=dict(required=False, type='bool'),
            pn_speed=dict(required=False, type='str',
                          choices=['disable', '10m', '100m',
                                   '1g', '2.5g', '10g', '25g',
                                   '40g', '50g', '100g']),
            pn_port=dict(required=False, type='str'),
            pn_vxlan_termination=dict(required=False, type='bool'),
            pn_pause=dict(required=False, type='bool'),
            pn_loopback=dict(required=False, type='bool'),
            pn_loop_vlans=dict(required=False, type='str'),
            pn_routing=dict(required=False, type='bool'),
            pn_edge_switch=dict(required=False, type='bool'),
            pn_enable=dict(required=False, type='bool'),
            pn_description=dict(required=False, type='str'),
            pn_host_enable=dict(required=False, type='bool'),
            pn_allowed_tpid=dict(required=False, type='str',
                                 choices=['vlan', 'q-in-q', 'q-in-q-old']),
            pn_mirror_only=dict(required=False, type='bool'),
            pn_reflect=dict(required=False, type='bool'),
            pn_jumbo=dict(required=False, type='bool'),
            pn_egress_rate_limit=dict(required=False, type='str'),
            pn_eth_mode=dict(required=False, type='str',
                             choices=['1000base-x', 'sgmii',
                                      'disabled', 'GMII']),
            pn_fabric_guard=dict(required=False, type='bool'),
            pn_local_switching=dict(required=False, type='bool'),
            pn_lacp_priority=dict(required=False, type='str'),
            pn_send_port=dict(required=False, type='str'),
            pn_port_mac_address=dict(required=False, type='str'),
            pn_defer_bringup=dict(required=False, type='bool'),
        ),
        required_if=(
            ['state', 'update', ['pn_port']],
        ),
        required_one_of=[['pn_intf', 'pn_crc_check_enable', 'pn_dscp_map',
                          'pn_speed', 'pn_autoneg',
                          'pn_vxlan_termination', 'pn_pause',
                          'pn_fec', 'pn_loopback', 'pn_loop_vlans',
                          'pn_routing', 'pn_edge_switch',
                          'pn_enable', 'pn_description',
                          'pn_host_enable', 'pn_allowed_tpid',
                          'pn_mirror_only', 'pn_reflect',
                          'pn_jumbo', 'pn_egress_rate_limit',
                          'pn_eth_mode', 'pn_fabric_guard',
                          'pn_local_switching', 'pn_lacp_priority',
                          'pn_send_port', 'pn_port_mac_address',
                          'pn_defer_bringup']],
    )

    # Accessing the arguments
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    intf = module.params['pn_intf']
    crc_check_enable = module.params['pn_crc_check_enable']
    dscp_map = module.params['pn_dscp_map']
    autoneg = module.params['pn_autoneg']
    speed = module.params['pn_speed']
    port = module.params['pn_port']
    vxlan_termination = module.params['pn_vxlan_termination']
    pause = module.params['pn_pause']
    loopback = module.params['pn_loopback']
    loop_vlans = module.params['pn_loop_vlans']
    routing = module.params['pn_routing']
    edge_switch = module.params['pn_edge_switch']
    enable = module.params['pn_enable']
    description = module.params['pn_description']
    host_enable = module.params['pn_host_enable']
    allowed_tpid = module.params['pn_allowed_tpid']
    mirror_only = module.params['pn_mirror_only']
    reflect = module.params['pn_reflect']
    jumbo = module.params['pn_jumbo']
    egress_rate_limit = module.params['pn_egress_rate_limit']
    eth_mode = module.params['pn_eth_mode']
    fabric_guard = module.params['pn_fabric_guard']
    local_switching = module.params['pn_local_switching']
    lacp_priority = module.params['pn_lacp_priority']
    send_port = module.params['pn_send_port']
    port_mac_address = module.params['pn_port_mac_address']
    defer_bringup = module.params['pn_defer_bringup']

    command = state_map[state]

    # Building the CLI command string
    cli = pn_cli(module, cliswitch)

    if dscp_map:
        NAME_EXISTS = check_cli(module, cli)

    if command == 'port-config-modify':
        cli += ' %s ' % command
        if dscp_map:
            if NAME_EXISTS is False:
                module.fail_json(
                    failed=True,
                    msg='Create dscp map with name %s before updating' % dscp_map
                )

            cli += ' dscp-map ' + dscp_map
        if intf:
            cli += ' intf ' + intf
        if speed:
            cli += ' speed ' + speed
        if port:
            cli += ' port ' + port
        if allowed_tpid:
            cli += ' allowed-tpid ' + allowed_tpid
        if egress_rate_limit:
            cli += ' egress-rate-limit ' + egress_rate_limit
        if eth_mode:
            cli += ' eth-mode ' + eth_mode
        if lacp_priority:
            cli += ' lacp-priority ' + lacp_priority
        if send_port:
            cli += ' send-port ' + send_port
        if port_mac_address:
            cli += ' port-mac-address ' + port_mac_address

        cli += booleanArgs(crc_check_enable, 'crc-check-enable', 'crc-check-disable')
        cli += booleanArgs(autoneg, 'autoneg', 'no-autoneg')
        cli += booleanArgs(vxlan_termination, 'vxlan-termination', 'no-vxlan-termination')
        cli += booleanArgs(pause, 'pause', 'no-pause')
        cli += booleanArgs(loopback, 'loopback', 'no-loopback')
        cli += booleanArgs(routing, 'routing', 'no-routing')
        cli += booleanArgs(edge_switch, 'edge-switch', 'no-edge-switch')
        cli += booleanArgs(enable, 'enable', 'disable')
        cli += booleanArgs(host_enable, 'host-enable', 'host-disable')
        cli += booleanArgs(mirror_only, 'mirror-only', 'no-mirror-receive-only')
        cli += booleanArgs(reflect, 'reflect', 'no-reflect')
        cli += booleanArgs(jumbo, 'jumbo', 'no-jumbo')
        cli += booleanArgs(fabric_guard, 'fabric-guard', 'no-fabric-guard')
        cli += booleanArgs(local_switching, 'local-switching', 'no-local-switching')
        cli += booleanArgs(defer_bringup, 'defer-bringup', 'no-defer-bringup')

    run_cli(module, cli, state_map)


if __name__ == '__main__':
    main()



# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible.module_utils._text import to_bytes

from ansible.parsing.vault import VaultSecret


class TextVaultSecret(VaultSecret):
    """A secret piece of text. ie, a password. Tracks text encoding.

    The text encoding of the text may not be the default text encoding so
    we keep track of the encoding so we encode it to the same bytes."""

    def __init__(self, text, encoding=None, errors=None, _bytes=None):
        super(TextVaultSecret, self).__init__()
        self.text = text
        self.encoding = encoding or "utf-8"
        self._bytes = _bytes
        self.errors = errors or "strict"

    @property
    def bytes(self):
        """The text encoded with encoding, unless we specifically set _bytes."""
        return self._bytes or to_bytes(
            self.text, encoding=self.encoding, errors=self.errors
        )



#!/usr/bin/python
# Copyright (c) 2016-2017 Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: oneview_ethernet_network_info
short_description: Retrieve the information about one or more of the OneView Ethernet Networks
description:
    - Retrieve the information about one or more of the Ethernet Networks from OneView.
    - This module was called C(oneview_ethernet_network_facts) before Ansible 2.9, returning C(ansible_facts).
      Note that the M(community.general.oneview_ethernet_network_info) module no longer returns C(ansible_facts)!
requirements:
    - hpOneView >= 2.0.1
author:
    - Felipe Bulsoni (@fgbulsoni)
    - Thiago Miotto (@tmiotto)
    - Adriane Cardozo (@adriane-cardozo)
options:
    name:
      description:
        - Ethernet Network name.
    options:
      description:
        - "List with options to gather additional information about an Ethernet Network and related resources.
          Options allowed: C(associatedProfiles) and C(associatedUplinkGroups)."
extends_documentation_fragment:
- community.general.oneview
- community.general.oneview.factsparams

'''

EXAMPLES = '''
- name: Gather information about all Ethernet Networks
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.ethernet_networks }}"

- name: Gather paginated and filtered information about Ethernet Networks
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
    params:
      start: 1
      count: 3
      sort: 'name:descending'
      filter: 'purpose=General'
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.ethernet_networks }}"

- name: Gather information about an Ethernet Network by name
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
    name: Ethernet network name
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.ethernet_networks }}"

- name: Gather information about an Ethernet Network by name with options
  community.general.oneview_ethernet_network_info:
    config: /etc/oneview/oneview_config.json
    name: eth1
    options:
      - associatedProfiles
      - associatedUplinkGroups
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.enet_associated_profiles }}"
- ansible.builtin.debug:
    msg: "{{ result.enet_associated_uplink_groups }}"
'''

RETURN = '''
ethernet_networks:
    description: Has all the OneView information about the Ethernet Networks.
    returned: Always, but can be null.
    type: dict

enet_associated_profiles:
    description: Has all the OneView information about the profiles which are using the Ethernet network.
    returned: When requested, but can be null.
    type: dict

enet_associated_uplink_groups:
    description: Has all the OneView information about the uplink sets which are using the Ethernet network.
    returned: When requested, but can be null.
    type: dict
'''

from ansible_collections.community.general.plugins.module_utils.oneview import OneViewModuleBase


class EthernetNetworkInfoModule(OneViewModuleBase):
    argument_spec = dict(
        name=dict(type='str'),
        options=dict(type='list'),
        params=dict(type='dict')
    )

    def __init__(self):
        super(EthernetNetworkInfoModule, self).__init__(additional_arg_spec=self.argument_spec)
        self.is_old_facts = self.module._name in ('oneview_ethernet_network_facts', 'community.general.oneview_ethernet_network_facts')
        if self.is_old_facts:
            self.module.deprecate("The 'oneview_ethernet_network_facts' module has been renamed to 'oneview_ethernet_network_info', "
                                  "and the renamed one no longer returns ansible_facts",
                                  version='3.0.0', collection_name='community.general')  # was Ansible 2.13

        self.resource_client = self.oneview_client.ethernet_networks

    def execute_module(self):
        info = {}
        if self.module.params['name']:
            ethernet_networks = self.resource_client.get_by('name', self.module.params['name'])

            if self.module.params.get('options') and ethernet_networks:
                info = self.__gather_optional_info(ethernet_networks[0])
        else:
            ethernet_networks = self.resource_client.get_all(**self.facts_params)

        info['ethernet_networks'] = ethernet_networks

        if self.is_old_facts:
            return dict(changed=False, ansible_facts=info)
        else:
            return dict(changed=False, **info)

    def __gather_optional_info(self, ethernet_network):

        info = {}

        if self.options.get('associatedProfiles'):
            info['enet_associated_profiles'] = self.__get_associated_profiles(ethernet_network)
        if self.options.get('associatedUplinkGroups'):
            info['enet_associated_uplink_groups'] = self.__get_associated_uplink_groups(ethernet_network)

        return info

    def __get_associated_profiles(self, ethernet_network):
        associated_profiles = self.resource_client.get_associated_profiles(ethernet_network['uri'])
        return [self.oneview_client.server_profiles.get(x) for x in associated_profiles]

    def __get_associated_uplink_groups(self, ethernet_network):
        uplink_groups = self.resource_client.get_associated_uplink_groups(ethernet_network['uri'])
        return [self.oneview_client.uplink_sets.get(x) for x in uplink_groups]


def main():
    EthernetNetworkInfoModule().run()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type
DOCUMENTATION = """
module: ios_l2_interface
extends_documentation_fragment:
- cisco.ios.ios
short_description: (deprecated, removed after 2022-06-01) Manage Layer-2 interface
  on Cisco IOS devices.
description:
- This module provides declarative management of Layer-2 interfaces on Cisco IOS devices.
version_added: 1.0.0
deprecated:
  alternative: ios_l2_interfaces
  why: Newer and updated modules released with more functionality in Ansible 2.9
  removed_at_date: '2022-06-01'
author:
- Nathaniel Case (@Qalthos)
options:
  name:
    description:
    - Full name of the interface excluding any logical unit number, i.e. GigabitEthernet0/1.
    aliases:
    - interface
    type: str
  mode:
    description:
    - Mode in which interface needs to be configured.
    choices:
    - access
    - trunk
    type: str
  access_vlan:
    description:
    - Configure given VLAN in access port. If C(mode=access), used as the access VLAN
      ID.
    type: str
  trunk_vlans:
    description:
    - List of VLANs to be configured in trunk port. If C(mode=trunk), used as the
      VLAN range to ADD or REMOVE from the trunk.
    type: str
  native_vlan:
    description:
    - Native VLAN to be configured in trunk port. If C(mode=trunk), used as the trunk
      native VLAN ID.
    type: str
  trunk_allowed_vlans:
    description:
    - List of allowed VLANs in a given trunk port. If C(mode=trunk), these are the
      only VLANs that will be configured on the trunk, i.e. "2-10,15".
    type: str
  aggregate:
    description:
    - List of Layer-2 interface definitions.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Full name of the interface excluding any logical unit number, i.e. GigabitEthernet0/1.
        aliases:
        - interface
        type: str
      mode:
        description:
        - Mode in which interface needs to be configured.
        choices:
        - access
        - trunk
        type: str
      access_vlan:
        description:
        - Configure given VLAN in access port. If C(mode=access), used as the access VLAN
          ID.
        type: str
      trunk_vlans:
        description:
        - List of VLANs to be configured in trunk port. If C(mode=trunk), used as the
          VLAN range to ADD or REMOVE from the trunk.
        type: str
      native_vlan:
        description:
        - Native VLAN to be configured in trunk port. If C(mode=trunk), used as the trunk
          native VLAN ID.
        type: str
      trunk_allowed_vlans:
        description:
        - List of allowed VLANs in a given trunk port. If C(mode=trunk), these are the
          only VLANs that will be configured on the trunk, i.e. "2-10,15".
        type: str
      state:
        description:
        - Manage the state of the Layer-2 Interface configuration.
        choices:
        - present
        - absent
        - unconfigured
        type: str
  state:
    description:
    - Manage the state of the Layer-2 Interface configuration.
    default: present
    choices:
    - present
    - absent
    - unconfigured
    type: str

"""
EXAMPLES = """
- name: Ensure GigabitEthernet0/5 is in its default l2 interface state
  ios.ios_l2_interface:
    name: GigabitEthernet0/5
    state: unconfigured
- name: Ensure GigabitEthernet0/5 is configured for access vlan 20
  ios.ios_l2_interface:
    name: GigabitEthernet0/5
    mode: access
    access_vlan: 20
- name: Ensure GigabitEthernet0/5 only has vlans 5-10 as trunk vlans
  ios.ios_l2_interface:
    name: GigabitEthernet0/5
    mode: trunk
    native_vlan: 10
    trunk_allowed_vlans: 5-10
- name: Ensure GigabitEthernet0/5 is a trunk port and ensure 2-50 are being tagged
    (doesn't mean others aren't also being tagged)
  ios.ios_l2_interface:
    name: GigabitEthernet0/5
    mode: trunk
    native_vlan: 10
    trunk_vlans: 2-50
- name: Ensure these VLANs are not being tagged on the trunk
  ios.ios_l2_interface:
    name: GigabitEthernet0/5
    mode: trunk
    trunk_vlans: 51-4094
    state: absent
"""
RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always, except for the platforms that use Netconf transport to manage the device.
  type: list
  sample:
    - interface GigabitEthernet0/5
    - switchport access vlan 20
"""
import re
from copy import deepcopy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    load_config,
    run_commands,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.ios import (
    ios_argument_spec,
)


def get_interface_type(interface):
    intf_type = "unknown"
    if interface.upper()[:2] in (
        "ET",
        "GI",
        "FA",
        "TE",
        "FO",
        "HU",
        "TWE",
        "TW",
    ):
        intf_type = "ethernet"
    elif interface.upper().startswith("VL"):
        intf_type = "svi"
    elif interface.upper().startswith("LO"):
        intf_type = "loopback"
    elif interface.upper()[:2] in ("MG", "MA"):
        intf_type = "management"
    elif interface.upper().startswith("PO"):
        intf_type = "portchannel"
    elif interface.upper().startswith("NV"):
        intf_type = "nve"
    return intf_type


def is_switchport(name, module):
    intf_type = get_interface_type(name)
    if intf_type in ("ethernet", "portchannel"):
        config = run_commands(
            module, ["show interface {0} switchport".format(name)]
        )[0]
        match = re.search("Switchport: Enabled", config)
        return bool(match)
    return False


def interface_is_portchannel(name, module):
    if get_interface_type(name) == "ethernet":
        config = run_commands(module, ["show run interface {0}".format(name)])[
            0
        ]
        if any(c in config for c in ["channel group", "channel-group"]):
            return True
    return False


def get_switchport(name, module):
    config = run_commands(
        module, ["show interface {0} switchport".format(name)]
    )[0]
    mode = re.search("Administrative Mode: (?:.* )?(\\w+)$", config, re.M)
    access = re.search("Access Mode VLAN: (\\d+)", config)
    native = re.search("Trunking Native Mode VLAN: (\\d+)", config)
    trunk = re.search("Trunking VLANs Enabled: (.+)$", config, re.M)
    if mode:
        mode = mode.group(1)
    if access:
        access = access.group(1)
    if native:
        native = native.group(1)
    if trunk:
        trunk = trunk.group(1)
    if trunk == "ALL":
        trunk = "1-4094"
    switchport_config = {
        "interface": name,
        "mode": mode,
        "access_vlan": access,
        "native_vlan": native,
        "trunk_vlans": trunk,
    }
    return switchport_config


def remove_switchport_config_commands(name, existing, proposed, module):
    mode = proposed.get("mode")
    commands = []
    command = None
    if mode == "access":
        av_check = existing.get("access_vlan") == proposed.get("access_vlan")
        if av_check:
            command = "no switchport access vlan {0}".format(
                existing.get("access_vlan")
            )
            commands.append(command)
    elif mode == "trunk":
        # Supported Remove Scenarios for trunk_vlans_list
        # 1) Existing: 1,2,3 Proposed: 1,2,3 - Remove all
        # 2) Existing: 1,2,3 Proposed: 1,2   - Remove 1,2 Leave 3
        # 3) Existing: 1,2,3 Proposed: 2,3   - Remove 2,3 Leave 1
        # 4) Existing: 1,2,3 Proposed: 4,5,6 - None removed.
        # 5) Existing: None  Proposed: 1,2,3 - None removed.
        existing_vlans = existing.get("trunk_vlans_list")
        proposed_vlans = proposed.get("trunk_vlans_list")
        vlans_to_remove = set(proposed_vlans).intersection(existing_vlans)
        if vlans_to_remove:
            proposed_allowed_vlans = proposed.get("trunk_allowed_vlans")
            remove_trunk_allowed_vlans = proposed.get(
                "trunk_vlans", proposed_allowed_vlans
            )
            command = "switchport trunk allowed vlan remove {0}".format(
                remove_trunk_allowed_vlans
            )
            commands.append(command)
        native_check = existing.get("native_vlan") == proposed.get(
            "native_vlan"
        )
        if native_check and proposed.get("native_vlan"):
            command = "no switchport trunk native vlan {0}".format(
                existing.get("native_vlan")
            )
            commands.append(command)
    if commands:
        commands.insert(0, "interface " + name)
    return commands


def get_switchport_config_commands(name, existing, proposed, module):
    """Gets commands required to config a given switchport interface
    """
    proposed_mode = proposed.get("mode")
    existing_mode = existing.get("mode")
    commands = []
    command = None
    if proposed_mode != existing_mode:
        if proposed_mode == "trunk":
            command = "switchport mode trunk"
        elif proposed_mode == "access":
            command = "switchport mode access"
    if command:
        commands.append(command)
    if proposed_mode == "access":
        av_check = str(existing.get("access_vlan")) == str(
            proposed.get("access_vlan")
        )
        if not av_check:
            command = "switchport access vlan {0}".format(
                proposed.get("access_vlan")
            )
            commands.append(command)
    elif proposed_mode == "trunk":
        tv_check = existing.get("trunk_vlans_list") == proposed.get(
            "trunk_vlans_list"
        )
        if not tv_check:
            if proposed.get("allowed"):
                command = "switchport trunk allowed vlan {0}".format(
                    proposed.get("trunk_allowed_vlans")
                )
                commands.append(command)
            else:
                existing_vlans = existing.get("trunk_vlans_list")
                proposed_vlans = proposed.get("trunk_vlans_list")
                vlans_to_add = set(proposed_vlans).difference(existing_vlans)
                if vlans_to_add:
                    command = "switchport trunk allowed vlan add {0}".format(
                        proposed.get("trunk_vlans")
                    )
                    commands.append(command)
        native_check = str(existing.get("native_vlan")) == str(
            proposed.get("native_vlan")
        )
        if not native_check and proposed.get("native_vlan"):
            command = "switchport trunk native vlan {0}".format(
                proposed.get("native_vlan")
            )
            commands.append(command)
    if commands:
        commands.insert(0, "interface " + name)
    return commands


def is_switchport_default(existing):
    """Determines if switchport has a default config based on mode
    Args:
        existing (dict): existing switchport configuration from Ansible mod
    Returns:
        boolean: True if switchport has OOB Layer 2 config, i.e.
           vlan 1 and trunk all and mode is access
    """
    c1 = str(existing["access_vlan"]) == "1"
    c2 = str(existing["native_vlan"]) == "1"
    c3 = existing["trunk_vlans"] == "1-4094"
    c4 = existing["mode"] == "access"
    default = c1 and c2 and c3 and c4
    return default


def default_switchport_config(name):
    commands = []
    commands.append("interface " + name)
    commands.append("switchport mode access")
    commands.append("switch access vlan 1")
    commands.append("switchport trunk native vlan 1")
    commands.append("switchport trunk allowed vlan all")
    return commands


def vlan_range_to_list(vlans):
    result = []
    if vlans:
        for part in vlans.split(","):
            if part.lower() == "none":
                break
            if part:
                if "-" in part:
                    start, stop = (int(i) for i in part.split("-"))
                    result.extend(range(start, stop + 1))
                else:
                    result.append(int(part))
    return sorted(result)


def get_list_of_vlans(module):
    config = run_commands(module, ["show vlan"])[0]
    vlans = set()
    lines = config.strip().splitlines()
    for line in lines:
        line_parts = line.split()
        if line_parts:
            try:
                int(line_parts[0])
            except ValueError:
                continue
            vlans.add(line_parts[0])
    return list(vlans)


def flatten_list(commands):
    flat_list = []
    for command in commands:
        if isinstance(command, list):
            flat_list.extend(command)
        else:
            flat_list.append(command)
    return flat_list


def map_params_to_obj(module):
    obj = []
    aggregate = module.params.get("aggregate")
    if aggregate:
        for item in aggregate:
            for key in item:
                if item.get(key) is None:
                    item[key] = module.params[key]
            obj.append(item.copy())
    else:
        obj.append(
            {
                "name": module.params["name"],
                "mode": module.params["mode"],
                "access_vlan": module.params["access_vlan"],
                "native_vlan": module.params["native_vlan"],
                "trunk_vlans": module.params["trunk_vlans"],
                "trunk_allowed_vlans": module.params["trunk_allowed_vlans"],
                "state": module.params["state"],
            }
        )
    return obj


def main():
    """ main entry point for module execution
    """
    element_spec = dict(
        name=dict(type="str", aliases=["interface"]),
        mode=dict(choices=["access", "trunk"]),
        access_vlan=dict(type="str"),
        native_vlan=dict(type="str"),
        trunk_vlans=dict(type="str"),
        trunk_allowed_vlans=dict(type="str"),
        state=dict(
            choices=["absent", "present", "unconfigured"], default="present"
        ),
    )
    aggregate_spec = deepcopy(element_spec)
    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)
    argument_spec = dict(
        aggregate=dict(type="list", elements="dict", options=aggregate_spec)
    )
    argument_spec.update(element_spec)
    argument_spec.update(ios_argument_spec)
    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["access_vlan", "trunk_vlans"],
            ["access_vlan", "native_vlan"],
            ["access_vlan", "trunk_allowed_vlans"],
        ],
        supports_check_mode=True,
    )
    warnings = list()
    commands = []
    result = {"changed": False, "warnings": warnings}
    want = map_params_to_obj(module)
    for w in want:
        name = w["name"]
        mode = w["mode"]
        access_vlan = w["access_vlan"]
        state = w["state"]
        trunk_vlans = w["trunk_vlans"]
        native_vlan = w["native_vlan"]
        trunk_allowed_vlans = w["trunk_allowed_vlans"]
        args = dict(
            name=name,
            mode=mode,
            access_vlan=access_vlan,
            native_vlan=native_vlan,
            trunk_vlans=trunk_vlans,
            trunk_allowed_vlans=trunk_allowed_vlans,
        )
        proposed = dict((k, v) for k, v in args.items() if v is not None)
        name = name.lower()
        if mode == "access" and state == "present" and not access_vlan:
            module.fail_json(
                msg="access_vlan param is required when mode=access && state=present"
            )
        if mode == "trunk" and access_vlan:
            module.fail_json(
                msg="access_vlan param not supported when using mode=trunk"
            )
        if not is_switchport(name, module):
            module.fail_json(
                msg="""Ensure interface is configured to be a L2
port first before using this module. You can use
the ios_interface module for this."""
            )
        if interface_is_portchannel(name, module):
            module.fail_json(
                msg="""Cannot change L2 config on physical
port because it is in a portchannel.
You should update the portchannel config."""
            )
        # existing will never be null for Eth intfs as there is always a default
        existing = get_switchport(name, module)
        # Safeguard check
        # If there isn't an existing, something is wrong per previous comment
        if not existing:
            module.fail_json(
                msg="Make sure you are using the FULL interface name"
            )
        if trunk_vlans or trunk_allowed_vlans:
            if trunk_vlans:
                trunk_vlans_list = vlan_range_to_list(trunk_vlans)
            elif trunk_allowed_vlans:
                trunk_vlans_list = vlan_range_to_list(trunk_allowed_vlans)
                proposed["allowed"] = True
            existing_trunks_list = vlan_range_to_list(existing["trunk_vlans"])
            existing["trunk_vlans_list"] = existing_trunks_list
            proposed["trunk_vlans_list"] = trunk_vlans_list
        current_vlans = get_list_of_vlans(module)
        if state == "present":
            if access_vlan and access_vlan not in current_vlans:
                module.fail_json(
                    msg="""You are trying to configure a VLAN on an interface that
does not exist on the  switch yet!""",
                    vlan=access_vlan,
                )
            elif native_vlan and native_vlan not in current_vlans:
                module.fail_json(
                    msg="""You are trying to configure a VLAN on an interface that
does not exist on the  switch yet!""",
                    vlan=native_vlan,
                )
            else:
                command = get_switchport_config_commands(
                    name, existing, proposed, module
                )
                commands.append(command)
        elif state == "unconfigured":
            is_default = is_switchport_default(existing)
            if not is_default:
                command = default_switchport_config(name)
                commands.append(command)
        elif state == "absent":
            command = remove_switchport_config_commands(
                name, existing, proposed, module
            )
            commands.append(command)
        if trunk_vlans or trunk_allowed_vlans:
            existing.pop("trunk_vlans_list")
            proposed.pop("trunk_vlans_list")
    cmds = flatten_list(commands)
    if cmds:
        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            result["changed"] = True
            load_config(module, cmds)
            if "configure" in cmds:
                cmds.pop(0)
    result["commands"] = cmds
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# (c) 2018, René Moser <mail@renemoser.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: cloud_init_data_facts
short_description: Retrieve facts of cloud-init.
description:
  - Gathers facts by reading the status.json and result.json of cloud-init.
author: René Moser (@resmo)
options:
  filter:
    description:
      - Filter facts
    type: str
    choices: [ status, result ]
notes:
  - See http://cloudinit.readthedocs.io/ for more information about cloud-init.
'''

EXAMPLES = '''
- name: Gather all facts of cloud init
  community.general.cloud_init_data_facts:
  register: result

- ansible.builtin.debug:
    var: result

- name: Wait for cloud init to finish
  community.general.cloud_init_data_facts:
    filter: status
  register: res
  until: "res.cloud_init_data_facts.status.v1.stage is defined and not res.cloud_init_data_facts.status.v1.stage"
  retries: 50
  delay: 5
'''

RETURN = '''
---
cloud_init_data_facts:
  description: Facts of result and status.
  returned: success
  type: dict
  sample: '{
    "status": {
        "v1": {
            "datasource": "DataSourceCloudStack",
            "errors": []
        },
    "result": {
        "v1": {
            "datasource": "DataSourceCloudStack",
            "init": {
                "errors": [],
                "finished": 1522066377.0185432,
                "start": 1522066375.2648022
            },
            "init-local": {
                "errors": [],
                "finished": 1522066373.70919,
                "start": 1522066373.4726632
            },
            "modules-config": {
                "errors": [],
                "finished": 1522066380.9097016,
                "start": 1522066379.0011985
            },
            "modules-final": {
                "errors": [],
                "finished": 1522066383.56594,
                "start": 1522066382.3449218
            },
            "stage": null
        }
    }'
'''

import os

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_text


CLOUD_INIT_PATH = "/var/lib/cloud/data/"


def gather_cloud_init_data_facts(module):
    res = {
        'cloud_init_data_facts': dict()
    }

    for i in ['result', 'status']:
        filter = module.params.get('filter')
        if filter is None or filter == i:
            res['cloud_init_data_facts'][i] = dict()
            json_file = CLOUD_INIT_PATH + i + '.json'

            if os.path.exists(json_file):
                f = open(json_file, 'rb')
                contents = to_text(f.read(), errors='surrogate_or_strict')
                f.close()

                if contents:
                    res['cloud_init_data_facts'][i] = module.from_json(contents)
    return res


def main():
    module = AnsibleModule(
        argument_spec=dict(
            filter=dict(choices=['result', 'status']),
        ),
        supports_check_mode=True,
    )

    facts = gather_cloud_init_data_facts(module)
    result = dict(changed=False, ansible_facts=facts, **facts)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)from __future__ import (absolute_import, division, print_function)
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json
import pytest
from ansible_collections.community.general.tests.unit.compat.mock import patch
from ansible_collections.community.general.plugins.modules.monitoring import pagerduty_change
from ansible_collections.community.general.tests.unit.plugins.modules.utils import AnsibleExitJson, AnsibleFailJson, ModuleTestCase, set_module_args


class TestPagerDutyChangeModule(ModuleTestCase):
    def setUp(self):
        super(TestPagerDutyChangeModule, self).setUp()
        self.module = pagerduty_change

    def tearDown(self):
        super(TestPagerDutyChangeModule, self).tearDown()

    @pytest.fixture
    def fetch_url_mock(self, mocker):
        return mocker.patch('ansible.module_utils.monitoring.pagerduty_change.fetch_url')

    def test_module_fail_when_required_args_missing(self):
        with self.assertRaises(AnsibleFailJson):
            set_module_args({})
            self.module.main()

    def test_ensure_change_event_created_with_minimal_data(self):
        set_module_args({
            'integration_key': 'test',
            'summary': 'Testing'
        })

        with patch.object(pagerduty_change, 'fetch_url') as fetch_url_mock:
            fetch_url_mock.return_value = (None, {"status": 202})
            with self.assertRaises(AnsibleExitJson):
                self.module.main()

            assert fetch_url_mock.call_count == 1
            url = fetch_url_mock.call_args[0][1]
            json_data = fetch_url_mock.call_args[1]['data']
            data = json.loads(json_data)

            assert url == 'https://events.pagerduty.com/v2/change/enqueue'
            assert data['routing_key'] == 'test'
            assert data['payload']['summary'] == 'Testing'
            assert data['payload']['source'] == 'Ansible'

    def test_ensure_change_event_created_with_full_data(self):
        set_module_args({
            'integration_key': 'test',
            'summary': 'Testing',
            'source': 'My Ansible Script',
            'user': 'ansible',
            'repo': 'github.com/ansible/ansible',
            'revision': '8c67432',
            'environment': 'production',
            'link_url': 'https://pagerduty.com',
            'link_text': 'PagerDuty'
        })

        with patch.object(pagerduty_change, 'fetch_url') as fetch_url_mock:
            fetch_url_mock.return_value = (None, {"status": 202})
            with self.assertRaises(AnsibleExitJson):
                self.module.main()

            assert fetch_url_mock.call_count == 1
            url = fetch_url_mock.call_args[0][1]
            json_data = fetch_url_mock.call_args[1]['data']
            data = json.loads(json_data)

            assert url == 'https://events.pagerduty.com/v2/change/enqueue'
            assert data['routing_key'] == 'test'
            assert data['payload']['summary'] == 'Testing'
            assert data['payload']['source'] == 'My Ansible Script'
            assert data['payload']['custom_details']['user'] == 'ansible'
            assert data['payload']['custom_details']['repo'] == 'github.com/ansible/ansible'
            assert data['payload']['custom_details']['revision'] == '8c67432'
            assert data['payload']['custom_details']['environment'] == 'production'
            assert data['links'][0]['href'] == 'https://pagerduty.com'
            assert data['links'][0]['text'] == 'PagerDuty'



#
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

"""
The cisco.ios_bgp_global config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.resource_module import (
    ResourceModule,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import (
    Facts,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.bgp_global import (
    Bgp_globalTemplate,
)


class Bgp_global(ResourceModule):
    """
    The cisco.ios_bgp_global config class
    """

    parsers = [
        "as_number",
        "bgp.additional_paths",
        "bgp.dampening",
        "bgp.graceful_shutdown",
        "route_server_context",
        "synchronization",
        "table_map",
        "timers",
    ]

    def __init__(self, module):
        super(Bgp_global, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="bgp_global",
            tmplt=Bgp_globalTemplate(),
        )

    def execute_module(self):
        """ Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """ Generate configuration commands to send based on
            want, have and desired state.
        """
        if self.want:
            wantd = {self.want["as_number"]: self.want}
        else:
            wantd = {}
        if self.have:
            haved = {self.have["as_number"]: self.have}
        else:
            haved = {}

        wantd, haved = self.list_to_dict(wantd, haved)
        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted" and self.have:
            if (
                not self.want
                or self.have.get("as_number") == self.want.get("as_number")
                and len(self.have) > 1
            ):
                self.addcmd(
                    {"as_number": haved[list(haved)[0]].pop("as_number")},
                    "as_number",
                    False,
                )
                self.compare(
                    parsers=self.parsers, want={}, have=haved[list(haved)[0]]
                )
                self._compare(want={}, have=haved[list(haved)[0]])
                self._list_params_compare(want={}, have=haved[list(haved)[0]])
            wantd = {}

        if self.state == "purged" and self.have:
            if (
                not self.want
                or (self.have.get("as_number") == self.want.get("as_number"))
                and len(self.have) >= 1
            ):
                self.addcmd(
                    {"as_number": haved[list(haved)[0]].pop("as_number")},
                    "as_number",
                    True,
                )
                wantd = {}

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
           populates the list of commands to be run by comparing
           the `want` and `have` data with the `parsers` defined
           for the Ospf_interfaces network resource.
        """
        if want != have and self.state != "deleted":
            self.addcmd(have, "as_number", False)
            self.compare(parsers=self.parsers, want=want, have=have)
            self._bgp_config_compare(want.get("bgp"), have.get("bgp"))
            self._list_params_compare(want, have)
        elif self.state == "deleted":
            self._bgp_config_compare(dict(), have.get("bgp"))

    def _bgp_config_compare(self, want, have):
        if want and have and want != have and isinstance(want, dict):
            set_have = True
            for k, val in iteritems(want):
                if isinstance(val, dict):
                    if k in have:
                        self.compare(
                            parsers=["bgp.config"],
                            want={"bgp": {k: val}},
                            have={"bgp": {k: have[k]}},
                        )
                    if k in have and self.state == "replaced":
                        if set_have:
                            self.compare(
                                parsers=["bgp.config"],
                                want=dict(),
                                have={"bgp": {k: have[k]}},
                            )
                        self.compare(
                            parsers=["bgp.config"],
                            want={"bgp": {k: val}},
                            have={"bgp": {k: have[k]}},
                        )
                    else:
                        self.compare(
                            parsers=["bgp.config"],
                            want={"bgp": {k: val}},
                            have=dict(),
                        )
        elif want and not have:
            for k, val in iteritems(want):
                if not isinstance(val, list):
                    self.compare(
                        parsers=["bgp.config"],
                        want={"bgp": {k: val}},
                        have=dict(),
                    )
        elif not want and have:
            for k, val in iteritems(have):
                if not isinstance(val, list):
                    self.compare(
                        parsers=["bgp.config"],
                        want=dict(),
                        have={"bgp": {k: val}},
                    )

    def _list_params_compare(self, want, have):
        def multi_compare(parser, want, have):
            if want:
                dict_iter = want
            else:
                dict_iter = have
            for k, v in iteritems(dict_iter):
                if parser == "neighbor":
                    type = None
                    if want.get("address") or have.get("address"):
                        type = "address"
                        want_type_val = want.get("address")
                        have_type_val = have.get("address")
                    if want.get("tag") or have.get("tag"):
                        type = "tag"
                        want_type_val = want.get("tag")
                        have_type_val = have.get("tag")
                    if want.get("ipv6_adddress") or have.get("ipv6_address"):
                        type = "ipv6_adddress"
                        want_type_val = want.get("ipv6_adddress")
                        have_type_val = have.get("ipv6_adddress")
                    if want and have:
                        self.compare(
                            parsers=[parser],
                            want={parser: {k: v, type: want_type_val}},
                            have={
                                parser: {
                                    k: have.get(k, {}),
                                    type: have_type_val,
                                }
                            },
                        )
                    elif not have:
                        self.compare(
                            parsers=[parser],
                            want={parser: {k: v, type: want_type_val}},
                            have=dict(),
                        )
                    elif not want:
                        self.compare(
                            parsers=[parser],
                            want=dict(),
                            have={parser: {k: v, type: have_type_val}},
                        )
                if parser == "redistribute":
                    if want and have:
                        self.compare(
                            parsers=[parser],
                            want={parser: {k: v}},
                            have={parser: {k: have.get(k, {})}},
                        )
                    elif not have:
                        self.compare(
                            parsers=[parser],
                            want={parser: {k: v}},
                            have=dict(),
                        )
                    elif not want:
                        self.compare(
                            parsers=[parser],
                            want=dict(),
                            have={parser: {k: v}},
                        )

        for every in ["bgp", "neighbor", "redistribute"]:

            param_want = want.get(every)
            param_have = have.get(every)
            if param_want and param_want != param_have:
                set_have = True
                if every == "bgp":
                    for each in ["bestpath", "nopeerup_delay"]:
                        set_have = True
                        for k, v in iteritems(param_want.get(each)):
                            if (
                                param_have
                                and k in param_have.get(each)
                                and self.state == "merged"
                            ):
                                if k in param_have.get(each):
                                    self.compare(
                                        parsers=[every + "." + each],
                                        want={"bgp": {each: {k: v}}},
                                        have={
                                            "bgp": {
                                                each: {
                                                    k: param_have.get(
                                                        each, {}
                                                    )[k]
                                                }
                                            }
                                        },
                                    )

                            elif param_have and self.state == "replaced":
                                if set_have and param_have.get(each):
                                    if isinstance(each, dict):
                                        for key_have, val_have in iteritems(
                                            param_have.get(each)
                                        ):
                                            multi_compare(
                                                parser=every,
                                                want=dict(),
                                                have=val_have,
                                            )
                                    else:
                                        # q(param_have, param_want)
                                        temp_have = {
                                            each: {i: param_have[each][i]}
                                            for i in list(param_have[each])
                                            if i not in param_want[each]
                                        }
                                        temp_want = {
                                            each: {i: param_want[each][i]}
                                            for i in list(param_want[each])
                                            if i not in param_have[each]
                                        }
                                        # q(temp_have)
                                        if temp_have:
                                            self.compare(
                                                parsers=[every + "." + each],
                                                want=dict(),
                                                have={"bgp": temp_have},
                                            )
                                        if temp_want:
                                            self.compare(
                                                parsers=[every + "." + each],
                                                want={"bgp": temp_want},
                                                have=dict(),
                                            )
                                    set_have = False
                            else:
                                self.compare(
                                    parsers=[every + "." + each],
                                    want={"bgp": {each: {k: v}}},
                                    have=dict(),
                                )
                if every == "neighbor" or every == "redistribute":
                    for k, v in iteritems(param_want):
                        if every == "neighbor":
                            if param_have and self.state == "merged":
                                multi_compare(
                                    parser=every,
                                    want=v,
                                    have=param_have.pop(k, {}),
                                )
                            elif param_have and self.state == "replaced":
                                if set_have:
                                    for key_have, val_have in iteritems(
                                        param_have
                                    ):
                                        multi_compare(
                                            parser=every,
                                            want=dict(),
                                            have=val_have,
                                        )
                                    set_have = False
                                multi_compare(
                                    parser=every, want=v, have=dict()
                                )
                            else:
                                multi_compare(
                                    parser=every, want=v, have=dict()
                                )
                            self.commands = (
                                [
                                    each
                                    for each in self.commands
                                    if "neighbor" not in each
                                ]
                                + [
                                    each
                                    for each in self.commands
                                    if "remote-as" in each
                                    and "neighbor" in each
                                ]
                                + [
                                    each
                                    for each in self.commands
                                    if "remote-as" not in each
                                    and "neighbor" in each
                                ]
                            )
                        elif every == "redistribute":
                            if param_have and self.state == "merged":
                                multi_compare(
                                    parser=every,
                                    want={k: v},
                                    have={k: param_have.pop(k, {})},
                                )
                            if param_have and self.state == "replaced":
                                if set_have:
                                    for key_have, val_have in iteritems(
                                        param_have
                                    ):
                                        multi_compare(
                                            parser=every,
                                            want=dict(),
                                            have=val_have,
                                        )
                                    set_have = False
                                multi_compare(
                                    parser=every, want={k: v}, have=dict()
                                )
                            else:
                                multi_compare(
                                    parser=every, want={k: v}, have=dict()
                                )
            elif param_have and self.state == "deleted":
                del_config_have = True
                if param_have:
                    for k, v in iteritems(param_have):
                        if every == "bgp" and del_config_have:
                            for each in ["bestpath", "nopeerup_delay"]:
                                for k, v in iteritems(param_have.get(each)):
                                    self.compare(
                                        parsers=[every + "." + each],
                                        want=dict(),
                                        have={"bgp": {each: {k: v}}},
                                    )
                            del_config_have = False
                        elif every == "neighbor":
                            multi_compare(parser=every, want=dict(), have=v)
                        elif every == "redistribute":
                            if param_have:
                                multi_compare(
                                    parser=every, want=dict(), have={k: v}
                                )

    def list_to_dict(self, wantd, haved):
        for thing in wantd, haved:
            if thing:
                for key, val in iteritems(thing):
                    for every in ["bgp", "neighbor", "redistribute"]:
                        value = val.get(every)
                        if value:
                            if isinstance(value, dict):
                                for k, v in iteritems(val.get(every)):
                                    if isinstance(v, list):
                                        temp = dict()
                                        temp[k] = {}
                                        for each in v:
                                            temp[k].update(each)
                                        val[every][k] = temp[k]
                            elif isinstance(value, list):
                                temp = dict()
                                temp[every] = {}
                                for each in value:
                                    if every == "neighbor":
                                        if each.get("address"):
                                            temp[every].update(
                                                {each.get("address"): each}
                                            )
                                        elif each.get("tag"):
                                            temp[every].update(
                                                {each.get("tag"): each}
                                            )
                                        elif each.get("ipv6_adddress"):
                                            temp[every].update(
                                                {
                                                    each.get(
                                                        "ipv6_adddress"
                                                    ): each
                                                }
                                            )
                                    elif every == "redistribute":
                                        temp[every].update(each)
                                val[every] = temp[every]
        return wantd, haved



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: helm_plugin
short_description: Manage Helm plugins
version_added: "1.0.0"
author:
  - Abhijeet Kasurde (@Akasurde)
requirements:
  - "helm (https://github.com/helm/helm/releases)"
description:
  -  Manages Helm plugins.
options:
  release_namespace:
    description:
      - Kubernetes namespace where the helm plugin should be installed.
    required: true
    type: str
    aliases: [ namespace ]

#Helm options
  state:
    description:
      - If C(state=present) the Helm plugin will be installed.
      - If C(state=absent) the Helm plugin will be removed.
    choices: [ absent, present ]
    default: present
    type: str
  plugin_name:
    description:
      - Name of Helm plugin.
      - Required only if C(state=absent).
    type: str
  plugin_path:
    description:
      - Plugin path to a plugin on your local file system or a url of a remote VCS repo.
      - If plugin path from file system is provided, make sure that tar is present on remote
        machine and not on Ansible controller.
      - Required only if C(state=present).
    type: str
extends_documentation_fragment:
  - community.kubernetes.helm_common_options
'''

EXAMPLES = r'''
- name: Install Helm env plugin
  community.kubernetes.helm_plugin:
    plugin_path: https://github.com/adamreese/helm-env
    state: present

- name: Install Helm plugin from local filesystem
  community.kubernetes.helm_plugin:
    plugin_path: https://domain/path/to/plugin.tar.gz
    state: present

- name: Remove Helm env plugin
  community.kubernetes.helm_plugin:
    plugin_name: env
    state: absent
'''

RETURN = r'''
stdout:
  type: str
  description: Full `helm` command stdout, in case you want to display it or examine the event log
  returned: always
  sample: ''
stderr:
  type: str
  description: Full `helm` command stderr, in case you want to display it or examine the event log
  returned: always
  sample: ''
command:
  type: str
  description: Full `helm` command built by this module, in case you want to re-run the command outside the module or debug a problem.
  returned: always
  sample: helm plugin list ...
msg:
  type: str
  description: Info about successful command
  returned: always
  sample: "Plugin installed successfully"
rc:
  type: int
  description: Helm plugin command return code
  returned: always
  sample: 1
'''

from ansible.module_utils.basic import AnsibleModule, env_fallback


def main():
    module = AnsibleModule(
        argument_spec=dict(
            binary_path=dict(type='path'),
            release_namespace=dict(type='str', required=True, aliases=['namespace']),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            plugin_path=dict(type='str',),
            plugin_name=dict(type='str',),
            # Helm options
            context=dict(type='str', aliases=['kube_context'], fallback=(env_fallback, ['K8S_AUTH_CONTEXT'])),
            kubeconfig=dict(type='path', aliases=['kubeconfig_path'], fallback=(env_fallback, ['K8S_AUTH_KUBECONFIG'])),
        ),
        supports_check_mode=True,
        required_if=[
            ("state", "present", ("plugin_path",)),
            ("state", "absent", ("plugin_name",)),
        ],
        mutually_exclusive=[
            ['plugin_name', 'plugin_path'],
        ],
    )

    bin_path = module.params.get('binary_path')
    release_namespace = module.params.get('release_namespace')
    state = module.params.get('state')

    # Helm options
    kube_context = module.params.get('context')
    kubeconfig_path = module.params.get('kubeconfig')

    if bin_path is not None:
        helm_cmd_common = bin_path
    else:
        helm_cmd_common = 'helm'

    helm_cmd_common = module.get_bin_path(helm_cmd_common, required=True)

    helm_cmd_common += " plugin"

    if kube_context is not None:
        helm_cmd_common += " --kube-context " + kube_context

    if kubeconfig_path is not None:
        helm_cmd_common += " --kubeconfig " + kubeconfig_path

    helm_cmd_common += " --namespace=" + release_namespace

    if state == 'present':
        helm_cmd_common += " install %s" % module.params.get('plugin_path')
        if not module.check_mode:
            rc, out, err = module.run_command(helm_cmd_common)
        else:
            rc, out, err = (0, '', '')

        if rc == 1 and 'plugin already exists' in err:
            module.exit_json(
                failed=False,
                changed=False,
                msg="Plugin already exists",
                command=helm_cmd_common,
                stdout=out,
                stderr=err,
                rc=rc
            )
        elif rc == 0:
            module.exit_json(
                failed=False,
                changed=True,
                msg="Plugin installed successfully",
                command=helm_cmd_common,
                stdout=out,
                stderr=err,
                rc=rc,
            )
        else:
            module.fail_json(
                msg="Failure when executing Helm command.",
                command=helm_cmd_common,
                stdout=out,
                stderr=err,
                rc=rc,
            )
    elif state == 'absent':
        plugin_name = module.params.get('plugin_name')
        helm_plugin_list = helm_cmd_common + " list"
        rc, out, err = module.run_command(helm_plugin_list)
        if rc != 0 or (out == '' and err == ''):
            module.fail_json(
                msg="Failed to get Helm plugin info",
                command=helm_plugin_list,
                stdout=out,
                stderr=err,
                rc=rc,
            )

        if out:
            found = False
            for line in out.splitlines():
                if line.startswith("NAME"):
                    continue
                name, dummy, dummy = line.split('\t', 3)
                name = name.strip()
                if name == plugin_name:
                    found = True
                    break
            if found:
                helm_uninstall_cmd = "%s uninstall %s" % (helm_cmd_common, plugin_name)
                if not module.check_mode:
                    rc, out, err = module.run_command(helm_uninstall_cmd)
                else:
                    rc, out, err = (0, '', '')

                if rc == 0:
                    module.exit_json(
                        changed=True,
                        msg="Plugin uninstalled successfully",
                        command=helm_uninstall_cmd,
                        stdout=out,
                        stderr=err,
                        rc=rc
                    )
                module.fail_json(
                    msg="Failed to get Helm plugin uninstall",
                    command=helm_uninstall_cmd,
                    stdout=out,
                    stderr=err,
                    rc=rc,
                )
            else:
                module.exit_json(
                    failed=False,
                    changed=False,
                    msg="Plugin not found or is already uninstalled",
                    command=helm_plugin_list,
                    stdout=out,
                    stderr=err,
                    rc=rc
                )


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# (c) 2015, Yannig Perre <yannig.perre@gmail.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

'''
Nagios livestatus inventory script. Before using this script, please
update nagios_livestatus.ini file.

Livestatus is a nagios/naemon/shinken module which let you retrieve
informations stored in the monitoring core.

This plugin inventory need livestatus API for python. Please install it
before using this script (apt/pip/yum/...).

Checkmk livestatus: https://mathias-kettner.de/checkmk_livestatus.html
Livestatus API: http://www.naemon.org/documentation/usersguide/livestatus.html
'''

import os
import re
import argparse
import sys

from ansible.module_utils.six.moves import configparser
import json

try:
    from mk_livestatus import Socket
except ImportError:
    sys.exit("Error: mk_livestatus is needed. Try something like: pip install python-mk-livestatus")


class NagiosLivestatusInventory(object):

    def parse_ini_file(self):
        config = configparser.SafeConfigParser()
        config.read(os.path.dirname(os.path.realpath(__file__)) + '/nagios_livestatus.ini')
        for section in config.sections():
            if not config.has_option(section, 'livestatus_uri'):
                continue

            # If fields_to_retrieve is not set, using default fields
            fields_to_retrieve = self.default_fields_to_retrieve
            if config.has_option(section, 'fields_to_retrieve'):
                fields_to_retrieve = [field.strip() for field in config.get(section, 'fields_to_retrieve').split(',')]
                fields_to_retrieve = tuple(fields_to_retrieve)

            # default section values
            section_values = {
                'var_prefix': 'livestatus_',
                'host_filter': None,
                'host_field': 'name',
                'group_field': 'groups'
            }
            for key, value in section_values.items():
                if config.has_option(section, key):
                    section_values[key] = config.get(section, key).strip()

            # Retrieving livestatus string connection
            livestatus_uri = config.get(section, 'livestatus_uri')
            backend_definition = None

            # Local unix socket
            unix_match = re.match('unix:(.*)', livestatus_uri)
            if unix_match is not None:
                backend_definition = {'connection': unix_match.group(1)}

            # Remote tcp connection
            tcp_match = re.match('tcp:(.*):([^:]*)', livestatus_uri)
            if tcp_match is not None:
                backend_definition = {'connection': (tcp_match.group(1), int(tcp_match.group(2)))}

            # No valid livestatus_uri => exiting
            if backend_definition is None:
                raise Exception('livestatus_uri field is invalid (%s). Expected: unix:/path/to/live or tcp:host:port' % livestatus_uri)

            # Updating backend_definition with current value
            backend_definition['name'] = section
            backend_definition['fields'] = fields_to_retrieve
            for key, value in section_values.items():
                backend_definition[key] = value

            self.backends.append(backend_definition)

    def parse_options(self):
        parser = argparse.ArgumentParser()
        parser.add_argument('--host', nargs=1)
        parser.add_argument('--list', action='store_true')
        parser.add_argument('--pretty', action='store_true')
        self.options = parser.parse_args()

    def add_host(self, hostname, group):
        if group not in self.result:
            self.result[group] = {}
            self.result[group]['hosts'] = []
        if hostname not in self.result[group]['hosts']:
            self.result[group]['hosts'].append(hostname)

    def query_backend(self, backend, host=None):
        '''Query a livestatus backend'''
        hosts_request = Socket(backend['connection']).hosts.columns(backend['host_field'], backend['group_field'])

        if backend['host_filter'] is not None:
            hosts_request = hosts_request.filter(backend['host_filter'])

        if host is not None:
            hosts_request = hosts_request.filter('name = ' + host[0])

        hosts_request._columns += backend['fields']

        hosts = hosts_request.call()
        for host in hosts:
            hostname = host[backend['host_field']]
            hostgroups = host[backend['group_field']]
            if not isinstance(hostgroups, list):
                hostgroups = [hostgroups]
            self.add_host(hostname, 'all')
            self.add_host(hostname, backend['name'])
            for group in hostgroups:
                self.add_host(hostname, group)
            for field in backend['fields']:
                var_name = backend['var_prefix'] + field
                if hostname not in self.result['_meta']['hostvars']:
                    self.result['_meta']['hostvars'][hostname] = {}
                self.result['_meta']['hostvars'][hostname][var_name] = host[field]

    def __init__(self):

        self.defaultgroup = 'group_all'
        self.default_fields_to_retrieve = ('address', 'alias', 'display_name', 'childs', 'parents')
        self.backends = []
        self.options = None

        self.parse_ini_file()
        self.parse_options()

        self.result = {}
        self.result['_meta'] = {}
        self.result['_meta']['hostvars'] = {}
        self.json_indent = None
        if self.options.pretty:
            self.json_indent = 2

        if len(self.backends) == 0:
            sys.exit("Error: Livestatus configuration is missing. See nagios_livestatus.ini.")

        for backend in self.backends:
            self.query_backend(backend, self.options.host)

        if self.options.host:
            print(json.dumps(self.result['_meta']['hostvars'][self.options.host[0]], indent=self.json_indent))
        elif self.options.list:
            print(json.dumps(self.result, indent=self.json_indent))
        else:
            sys.exit("usage: --list or --host HOSTNAME [--pretty]")


NagiosLivestatusInventory()



# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import pytest
from units.modules.utils import set_module_args, exit_json, fail_json, AnsibleExitJson

from ansible.module_utils import basic
from ansible_collections.check_point.mgmt.plugins.modules import cp_mgmt_threat_rule_facts

OBJECT = {
    "from": 1,
    "to": 1,
    "total": 6,
    "objects": [
        "53de74b7-8f19-4cbe-99fc-a81ef0759bad"
    ]
}

SHOW_PLURAL_PAYLOAD = {
    'limit': 1,
    'details_level': 'uid'
}

SHOW_SINGLE_PAYLOAD = {
    'name': 'object_which_is_not_exist'
}

api_call_object = 'threat-rule'
api_call_object_plural_version = 'threat-rulebase'
failure_msg = '''{u'message': u'Requested object [object_which_is_not_exist] not found', u'code': u'generic_err_object_not_found'}'''


class TestCheckpointThreatRuleFacts(object):
    module = cp_mgmt_threat_rule_facts

    @pytest.fixture(autouse=True)
    def module_mock(self, mocker):
        return mocker.patch.multiple(basic.AnsibleModule, exit_json=exit_json, fail_json=fail_json)

    @pytest.fixture
    def connection_mock(self, mocker):
        connection_class_mock = mocker.patch('ansible.module_utils.network.checkpoint.checkpoint.Connection')
        return connection_class_mock.return_value

    def test_show_single_object_which_is_not_exist(self, mocker, connection_mock):
        connection_mock.send_request.return_value = (404, failure_msg)
        try:
            result = self._run_module(SHOW_SINGLE_PAYLOAD)
        except Exception as e:
            result = e.args[0]

        assert result['failed']
        assert 'Checkpoint device returned error 404 with message ' + failure_msg == result['msg']

    def test_show_few_objects(self, mocker, connection_mock):
        connection_mock.send_request.return_value = (200, OBJECT)
        result = self._run_module(SHOW_PLURAL_PAYLOAD)

        assert not result['changed']
        assert OBJECT == result['ansible_facts'][api_call_object_plural_version]

    def _run_module(self, module_args):
        set_module_args(module_args)
        with pytest.raises(AnsibleExitJson) as ex:
            self.module.main()
        return ex.value.args[0]



# -*- coding: utf-8 -*-
#
# (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import, division, print_function
__metaclass__ = type


# THIS FILE IS FOR COMPATIBILITY ONLY! YOU SHALL NOT IMPORT IT!
#
# This fill will be removed eventually, so if you're using it,
# please stop doing so.

from .basic import (
    HAS_PYOPENSSL,
    CRYPTOGRAPHY_HAS_X25519,
    CRYPTOGRAPHY_HAS_X25519_FULL,
    CRYPTOGRAPHY_HAS_X448,
    CRYPTOGRAPHY_HAS_ED25519,
    CRYPTOGRAPHY_HAS_ED448,
    HAS_CRYPTOGRAPHY,
    OpenSSLObjectError,
    OpenSSLBadPassphraseError,
)

from .cryptography_crl import (
    REVOCATION_REASON_MAP,
    REVOCATION_REASON_MAP_INVERSE,
    cryptography_decode_revoked_certificate,
)

from .cryptography_support import (
    cryptography_get_extensions_from_cert,
    cryptography_get_extensions_from_csr,
    cryptography_name_to_oid,
    cryptography_oid_to_name,
    cryptography_get_name,
    cryptography_decode_name,
    cryptography_parse_key_usage_params,
    cryptography_get_basic_constraints,
    cryptography_key_needs_digest_for_signing,
    cryptography_compare_public_keys,
)

from .pem import (
    identify_private_key_format,
)

from .math import (
    binary_exp_mod,
    simple_gcd,
    quick_is_not_prime,
    count_bits,
)

from ._obj2txt import obj2txt as _obj2txt

from ._objects_data import OID_MAP as _OID_MAP

from ._objects import OID_LOOKUP as _OID_LOOKUP
from ._objects import NORMALIZE_NAMES as _NORMALIZE_NAMES
from ._objects import NORMALIZE_NAMES_SHORT as _NORMALIZE_NAMES_SHORT

from .pyopenssl_support import (
    pyopenssl_normalize_name,
    pyopenssl_get_extensions_from_cert,
    pyopenssl_get_extensions_from_csr,
)

from .support import (
    get_fingerprint_of_bytes,
    get_fingerprint,
    load_privatekey,
    load_certificate,
    load_certificate_request,
    parse_name_field,
    convert_relative_to_datetime,
    get_relative_time_option,
    select_message_digest,
    OpenSSLObject,
)

from ..io import (
    load_file_if_exists,
    write_file,
)



#!/usr/bin/python
#
# Copyright (c) 2018 Zim Kalinowski, <zikalino@microsoft.com>
# Copyright (c) 2019 Matti Ranta, (@techknowlogick)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_mariadbfirewallrule
version_added: "0.1.2"
short_description: Manage MariaDB firewall rule instance
description:
    - Create, update and delete instance of MariaDB firewall rule.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
    server_name:
        description:
            - The name of the server.
        required: True
    name:
        description:
            - The name of the MariaDB firewall rule.
        required: True
    start_ip_address:
        description:
            - The start IP address of the MariaDB firewall rule. Must be IPv4 format.
    end_ip_address:
        description:
            - The end IP address of the MariaDB firewall rule. Must be IPv4 format.
    state:
        description:
            - Assert the state of the MariaDB firewall rule. Use C(present) to create or update a rule and C(absent) to ensure it is not present.
        default: present
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zim Kalinowski (@zikalino)
    - Matti Ranta (@techknowlogick)

'''

EXAMPLES = '''
  - name: Create (or update) MariaDB firewall rule
    azure_rm_mariadbfirewallrule:
      resource_group: myResourceGroup
      server_name: testserver
      name: rule1
      start_ip_address: 10.0.0.17
      end_ip_address: 10.0.0.20
'''

RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.DBforMariaDB/servers/testserver/fire
             wallRules/rule1"
'''

import time

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from msrestazure.azure_exceptions import CloudError
    from msrest.polling import LROPoller
    from azure.mgmt.rdbms.mariadb import MariaDBManagementClient
    from msrest.serialization import Model
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMMariaDbFirewallRule(AzureRMModuleBase):
    """Configuration class for an Azure RM MariaDB firewall rule resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            start_ip_address=dict(
                type='str'
            ),
            end_ip_address=dict(
                type='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.server_name = None
        self.name = None
        self.start_ip_address = None
        self.end_ip_address = None

        self.results = dict(changed=False)
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMMariaDbFirewallRule, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                         supports_check_mode=True,
                                                         supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        old_response = None
        response = None

        resource_group = self.get_resource_group(self.resource_group)

        old_response = self.get_firewallrule()

        if not old_response:
            self.log("MariaDB firewall rule instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("MariaDB firewall rule instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.log("Need to check if MariaDB firewall rule instance has to be deleted or may be updated")
                if (self.start_ip_address is not None) and (self.start_ip_address != old_response['start_ip_address']):
                    self.to_do = Actions.Update
                if (self.end_ip_address is not None) and (self.end_ip_address != old_response['end_ip_address']):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the MariaDB firewall rule instance")

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_update_firewallrule()

            if not old_response:
                self.results['changed'] = True
            else:
                self.results['changed'] = old_response.__ne__(response)
            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("MariaDB firewall rule instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_firewallrule()
            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_firewallrule():
                time.sleep(20)
        else:
            self.log("MariaDB firewall rule instance unchanged")
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]

        return self.results

    def create_update_firewallrule(self):
        '''
        Creates or updates MariaDB firewall rule with the specified configuration.

        :return: deserialized MariaDB firewall rule instance state dictionary
        '''
        self.log("Creating / Updating the MariaDB firewall rule instance {0}".format(self.name))

        try:
            response = self.mariadb_client.firewall_rules.create_or_update(resource_group_name=self.resource_group,
                                                                           server_name=self.server_name,
                                                                           firewall_rule_name=self.name,
                                                                           start_ip_address=self.start_ip_address,
                                                                           end_ip_address=self.end_ip_address)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except CloudError as exc:
            self.log('Error attempting to create the MariaDB firewall rule instance.')
            self.fail("Error creating the MariaDB firewall rule instance: {0}".format(str(exc)))
        return response.as_dict()

    def delete_firewallrule(self):
        '''
        Deletes specified MariaDB firewall rule instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the MariaDB firewall rule instance {0}".format(self.name))
        try:
            response = self.mariadb_client.firewall_rules.delete(resource_group_name=self.resource_group,
                                                                 server_name=self.server_name,
                                                                 firewall_rule_name=self.name)
        except CloudError as e:
            self.log('Error attempting to delete the MariaDB firewall rule instance.')
            self.fail("Error deleting the MariaDB firewall rule instance: {0}".format(str(e)))

        return True

    def get_firewallrule(self):
        '''
        Gets the properties of the specified MariaDB firewall rule.

        :return: deserialized MariaDB firewall rule instance state dictionary
        '''
        self.log("Checking if the MariaDB firewall rule instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mariadb_client.firewall_rules.get(resource_group_name=self.resource_group,
                                                              server_name=self.server_name,
                                                              firewall_rule_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("MariaDB firewall rule instance : {0} found".format(response.name))
        except CloudError as e:
            self.log('Did not find the MariaDB firewall rule instance.')
        if found is True:
            return response.as_dict()

        return False


def main():
    """Main execution"""
    AzureRMMariaDbFirewallRule()


if __name__ == '__main__':
    main()



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
from ansible_collections.cisco.nxos.plugins.modules import nxos_pim_interface
from .nxos_module import TestNxosModule, load_fixture, set_module_args


class TestNxosIPInterfaceModule(TestNxosModule):

    module = nxos_pim_interface

    def setUp(self):
        super(TestNxosIPInterfaceModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_pim_interface.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_pim_interface.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_run_commands = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_pim_interface.run_commands"
        )
        self.run_commands = self.mock_run_commands.start()

    def tearDown(self):
        super(TestNxosIPInterfaceModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_run_commands.stop()

    def load_fixtures(self, commands=None, device=""):
        module_name = self.module.__name__.rsplit(".", 1)[1]

        def load_from_file(*args, **kwargs):
            module, commands = args
            output = list()

            for command in commands:
                if type(command) == dict:
                    command = command["command"]
                filename = (
                    str(command)
                    .split(" | ")[0]
                    .replace(" ", "_")
                    .replace("/", "_")
                )
                output.append(load_fixture(module_name, filename))
            return output

        self.get_config.return_value = load_fixture(module_name, "config.cfg")
        self.load_config.return_value = None
        self.run_commands.side_effect = load_from_file

    def test_nxos_pim_interface_present(self):
        set_module_args(
            dict(
                interface="eth2/1",
                dr_prio=10,
                hello_interval=40,
                sparse=True,
                border=False,
            )
        )
        self.execute_module(
            changed=True,
            commands=[
                "interface eth2/1",
                "ip pim dr-priority 10",
                "ip pim hello-interval 40000",
                "ip pim sparse-mode",
            ],
        )

    def test_nxos_pim_interface_jp(self):
        set_module_args(
            dict(
                interface="eth2/1",
                jp_policy_in="JPIN",
                jp_policy_out="JPOUT",
                jp_type_in="routemap",
                jp_type_out="routemap",
            )
        )
        self.execute_module(
            changed=True,
            commands=[
                "interface eth2/1",
                "ip pim jp-policy JPOUT out",
                "ip pim jp-policy JPIN in",
            ],
        )

    def test_nxos_pim_interface_default(self):
        set_module_args(dict(interface="eth2/1", state="default"))
        self.execute_module(changed=False, commands=[])

    def test_nxos_pim_interface_ip_absent(self):
        set_module_args(dict(interface="eth2/1", state="absent"))
        self.execute_module(changed=False, commands=[])


class TestNxosPimInterfaceBfdModule(TestNxosModule):

    module = nxos_pim_interface

    def setUp(self):
        super(TestNxosPimInterfaceBfdModule, self).setUp()

        self.mock_get_interface_mode = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_pim_interface.get_interface_mode"
        )
        self.get_interface_mode = self.mock_get_interface_mode.start()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_pim_interface.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_pim_interface.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_run_commands = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_pim_interface.run_commands"
        )
        self.run_commands = self.mock_run_commands.start()

    def tearDown(self):
        super(TestNxosPimInterfaceBfdModule, self).tearDown()
        self.mock_get_interface_mode.stop()
        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_run_commands.stop()

    def load_fixtures(self, commands=None, device=""):
        self.load_config.return_value = None

    def test_bfd_1(self):
        # default (None) -> enable
        self.get_config.return_value = None
        set_module_args(dict(interface="eth2/1", bfd="enable"))
        self.execute_module(
            changed=True, commands=["interface eth2/1", "ip pim bfd-instance"]
        )

        # default (None) -> disable
        set_module_args(dict(interface="eth2/1", bfd="disable"))
        self.execute_module(
            changed=True,
            commands=["interface eth2/1", "ip pim bfd-instance disable"],
        )

        # default (None) -> default (None) (idempotence)
        set_module_args(dict(interface="eth2/1", bfd="default"))
        self.execute_module(changed=False)

        # default (None) -> interface state 'default'
        set_module_args(dict(interface="Ethernet9/3", state="default"))
        self.execute_module(changed=False)

        # default (None) -> interface state 'absent'
        set_module_args(dict(interface="Ethernet9/3", state="absent"))
        self.execute_module(changed=False)

    def test_bfd_2(self):
        # From disable
        self.get_config.return_value = """
            interface Ethernet9/2
              ip pim bfd-instance disable
        """
        # disable -> enable
        set_module_args(dict(interface="Ethernet9/2", bfd="enable"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/2", "ip pim bfd-instance"],
        )

        # disable -> disable (idempotence)
        set_module_args(dict(interface="Ethernet9/2", bfd="disable"))
        self.execute_module(changed=False)

        # disable -> default (None)
        set_module_args(dict(interface="Ethernet9/2", bfd="default"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/2", "no ip pim bfd-instance"],
        )
        # disable -> interface state 'default'
        set_module_args(dict(interface="Ethernet9/3", state="default"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/3", "no ip pim bfd-instance"],
        )

        # disable -> interface state 'absent'
        set_module_args(dict(interface="Ethernet9/3", state="absent"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/3", "no ip pim bfd-instance"],
        )

    def test_bfd_3(self):
        # From enable
        self.get_config.return_value = """
            interface Ethernet9/2
              ip pim bfd-instance
        """
        # enable -> disabled
        set_module_args(dict(interface="Ethernet9/3", bfd="disable"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/3", "ip pim bfd-instance disable"],
        )

        # enable -> enable (idempotence)
        set_module_args(dict(interface="Ethernet9/3", bfd="enable"))
        self.execute_module(changed=False)

        # enable -> default (None)
        set_module_args(dict(interface="Ethernet9/3", bfd="default"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/3", "no ip pim bfd-instance"],
        )

        # enable -> interface state 'default'
        set_module_args(dict(interface="Ethernet9/3", state="default"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/3", "no ip pim bfd-instance"],
        )

        # enable -> interface state 'absent'
        set_module_args(dict(interface="Ethernet9/3", state="absent"))
        self.execute_module(
            changed=True,
            commands=["interface Ethernet9/3", "no ip pim bfd-instance"],
        )
